@@ -90,7 +90,7 @@ void ActionDialog::controlInfoLabel(Device *dev)
     DeviceOptions opts;
     if (Device::AudioCd==dev->devType()) {
         opts.load(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
-    } else {
+    } else if (sourceUdi.isEmpty()) {
         opts=dev->options();
     }
 
@@ -144,7 +144,9 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
     // check space...
     haveVariousArtists=false;
     qint64 spaceRequired=0;
+    #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
     totalTime=0;
+    #endif
     foreach (const Song &s, songsToAction) {
         if (s.size>0) {
             spaceRequired+=s.size;
@@ -152,7 +154,9 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
         if (!haveVariousArtists && s.isVariousArtists()) {
             haveVariousArtists=true;
         }
+        #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
         totalTime+=s.time;
+        #endif
     }
 
     qint64 spaceAvailable=0;
@@ -261,8 +265,10 @@ void ActionDialog::init(const QString &srcUdi, const QString &dstUdi, const QLis
     paused=false;
     actionedSongs.clear();
     skippedSongs.clear();
+    #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
     actionedTime=0;
     timeTaken=0;
+    #endif
     currentPercent=0;
     currentDev=0;
     count=0;
@@ -285,7 +291,9 @@ void ActionDialog::slotButtonClicked(int button)
             }
             Settings::self()->saveOverwriteSongs(overwrite->isChecked());
             setPage(PAGE_PROGRESS);
+            #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
             timer.start();
+            #endif
             doNext();
             break;
         case Cancel:
@@ -300,13 +308,17 @@ void ActionDialog::slotButtonClicked(int button)
         }
         break;
     case PAGE_SKIP:
+        #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
         timeTaken+=timer.elapsed();
+        #endif
         setPage(PAGE_PROGRESS);
         switch(button) {
         case User1:
             skippedSongs.append(currentSong);
+            #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
             totalTime-=currentSong.time;
             timer.restart();
+            #endif
             incProgress();
             doNext();
             break;
@@ -472,7 +484,9 @@ void ActionDialog::actionStatus(int status, bool copiedCover)
     bool wasSkip=false;
     if (Device::Ok!=status && Device::NotConnected!=status && autoSkip) {
         skippedSongs.append(currentSong);
+        #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
         totalTime-=currentSong.time;
+        #endif
         wasSkip=true;
         status=Device::Ok;
     }
@@ -482,7 +496,9 @@ void ActionDialog::actionStatus(int status, bool copiedCover)
         if (Device::Ok==origStatus) {
             if (!wasSkip) {
                 actionedSongs.append(currentSong);
+                #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
                 actionedTime+=currentSong.time;
+                #endif
             }
             if (copiedCover) {
                 copiedCovers.insert(Utils::getDir(destFile));
@@ -672,6 +688,7 @@ QString ActionDialog::formatSong(const Song &s, bool showFiles, bool showTime)
         }
     }
 
+    #ifdef ACTION_DIALOG_SHOW_TIME_REMAINING
     if (showTime) {
         QString estimate=i18n("Unknown");
 
@@ -685,7 +702,10 @@ QString ActionDialog::formatSong(const Song &s, bool showFiles, bool showTime)
         str+=i18n("<tr><i><td align=\"right\"><i>Time remaining:</i></td><td><i>%5</i></td></i></tr>")
                 .arg(estimate);
     }
-
+    #else
+    Q_UNUSED(showTime)
+    #endif
+    
     return str+"</table>";
 }
 
