@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,6 +22,8 @@
  */
 
 #include "devicespage.h"
+#include "musiclibraryitemroot.h"
+#include "musiclibraryitemartist.h"
 #include "musiclibraryitemalbum.h"
 #include "musiclibraryitemsong.h"
 #include "mainwindow.h"
@@ -30,11 +32,9 @@
 #include "messagebox.h"
 #include "localize.h"
 #include "icons.h"
-#include "mainwindow.h"
 #include "action.h"
 #include "actioncollection.h"
-#include <QtGui/QIcon>
-#include <QtGui/QToolButton>
+#include "stdactions.h"
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobalSettings>
 #endif
@@ -49,35 +49,37 @@
 #include "tageditor.h"
 #include "actiondialog.h"
 #include "trackorganiser.h"
+#include "preferencesdialog.h"
+#include "coverdialog.h"
+#include "mpdconnection.h"
+#if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+#include "audiocddevice.h"
+#include "albumdetailsdialog.h"
+#include "cddbselectiondialog.h"
+#endif
 
-DevicesPage::DevicesPage(MainWindow *p)
+DevicesPage::DevicesPage(QWidget *p)
     : QWidget(p)
-    , mw(p)
 {
     setupUi(this);
-
-    configureAction = ActionCollection::get()->createAction("configuredevice", i18n("Configure Device"), Icons::configureIcon);
-    refreshAction = ActionCollection::get()->createAction("refreshdevice", i18n("Refresh Device"), "view-refresh");
-    copyAction = ActionCollection::get()->createAction("copytolibrary", i18n("Copy To Library"), "document-import");
+    copyAction = ActionCollection::get()->createAction("copytolibrary", i18n("Copy To Library"), Icons::importIcon);
     copyToLibraryButton->setDefaultAction(copyAction);
     syncAction = ActionCollection::get()->createAction("syncdevice", i18n("Sync"), "folder-sync");
     connect(syncAction, SIGNAL(triggered()), this, SLOT(sync()));
     #ifdef ENABLE_REMOTE_DEVICES
     forgetDeviceAction=ActionCollection::get()->createAction("forgetdevice", i18n("Forget Device"), "list-remove");
-    toggleDeviceAction=ActionCollection::get()->createAction("toggledevice", i18n("Toggle Device"), "network-connect");
     connect(forgetDeviceAction, SIGNAL(triggered()), this, SLOT(forgetRemoteDevice()));
-    connect(toggleDeviceAction, SIGNAL(triggered()), this, SLOT(toggleDevice()));
     #endif
-    Icon::init(copyToLibraryButton);
+    connect(DevicesModel::self()->connectAct(), SIGNAL(triggered()), this, SLOT(toggleDevice()));
+    connect(DevicesModel::self()->disconnectAct(), SIGNAL(triggered()), this, SLOT(toggleDevice()));
     copyToLibraryButton->setEnabled(false);
     syncAction->setEnabled(false);
     view->addAction(copyAction);
     view->addAction(syncAction);
-//     view->addAction(p->burnAction);
-    view->addAction(p->organiseFilesAction);
-    view->addAction(p->editTagsAction);
+    view->addAction(StdActions::self()->organiseFilesAction);
+    view->addAction(StdActions::self()->editTagsAction);
     #ifdef ENABLE_REPLAYGAIN_SUPPORT
-    view->addAction(p->replaygainAction);
+    view->addAction(StdActions::self()->replaygainAction);
     #endif
     #ifdef ENABLE_REMOTE_DEVICES
     QAction *sepA=new QAction(this);
@@ -88,21 +90,25 @@ DevicesPage::DevicesPage(MainWindow *p)
     QAction *sep=new QAction(this);
     sep->setSeparator(true);
     view->addAction(sep);
-    view->addAction(p->deleteSongsAction);
-    connect(DevicesModel::self(), SIGNAL(updateGenres(const QSet<QString> &)), this, SLOT(updateGenres(const QSet<QString> &)));
+    view->addAction(StdActions::self()->deleteSongsAction);
+    connect(this, SIGNAL(add(const QStringList &, bool, quint8)), MPDConnection::self(), SLOT(add(const QStringList &, bool, quint8)));
+    connect(this, SIGNAL(addSongsToPlaylist(const QString &, const QStringList &)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList &)));
     connect(genreCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(searchItems()));
+    connect(DevicesModel::self(), SIGNAL(updateGenres(const QSet<QString> &)), genreCombo, SLOT(update(const QSet<QString> &)));
+    connect(DevicesModel::self(), SIGNAL(updated(QModelIndex)), this, SLOT(updated(QModelIndex)));
     connect(view, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(itemDoubleClicked(const QModelIndex &)));
     connect(view, SIGNAL(searchItems()), this, SLOT(searchItems()));
     connect(view, SIGNAL(itemsSelected(bool)), SLOT(controlActions()));
+    connect(view, SIGNAL(rootIndexSet(QModelIndex)), this, SLOT(updateGenres(QModelIndex)));
     connect(copyAction, SIGNAL(triggered()), this, SLOT(copyToLibrary()));
-    connect(configureAction, SIGNAL(triggered()), this, SLOT(configureDevice()));
-    connect(refreshAction, SIGNAL(triggered()), this, SLOT(refreshDevice()));
-    Icon::init(menuButton);
-    menuButton->setPopupMode(QToolButton::InstantPopup);
+    connect(DevicesModel::self()->configureAct(), SIGNAL(triggered()), this, SLOT(configureDevice()));
+    connect(DevicesModel::self()->refreshAct(), SIGNAL(triggered()), this, SLOT(refreshDevice()));
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    connect(DevicesModel::self()->editAct(), SIGNAL(triggered()), this, SLOT(editDetails()));
+    connect(DevicesModel::self(), SIGNAL(matches(const QString &, const QList<CdAlbum> &)),
+            SLOT(cdMatches(const QString &, const QList<CdAlbum> &)));
+    #endif
     QMenu *menu=new QMenu(this);
-    menu->addAction(configureAction);
-    menu->addAction(refreshAction);
-    menu->addSeparator();
     #ifdef ENABLE_REMOTE_DEVICES
     Action *addRemote=ActionCollection::get()->createAction("adddevice", i18n("Add Device"), "network-server");
     connect(addRemote, SIGNAL(triggered()), this, SLOT(addRemoteDevice()));
@@ -110,27 +116,19 @@ DevicesPage::DevicesPage(MainWindow *p)
     menu->addAction(forgetDeviceAction);
     menu->addSeparator();
     #endif
-//     menu->addAction(copyAction);
-    menu->addAction(p->organiseFilesAction);
-    menu->addAction(p->editTagsAction);
+    menu->addAction(DevicesModel::self()->configureAct());
+    menu->addAction(DevicesModel::self()->refreshAct());
+    menu->addSeparator();
+    menu->addAction(StdActions::self()->organiseFilesAction);
+    menu->addAction(StdActions::self()->editTagsAction);
     #ifdef ENABLE_REPLAYGAIN_SUPPORT
-    menu->addAction(p->replaygainAction);
+    menu->addAction(StdActions::self()->replaygainAction);
     #endif
-    //menu->addAction(sep);
-    //menu->addAction(p->deleteSongsAction);
     menuButton->setMenu(menu);
-    menuButton->setIcon(Icons::menuIcon);
-    menuButton->setToolTip(i18n("Other Actions"));
     proxy.setSourceModel(DevicesModel::self());
     view->setTopText(i18n("Devices"));
     view->setModel(&proxy);
-    #ifdef ENABLE_REMOTE_DEVICES
-    view->init(configureAction, refreshAction, toggleDeviceAction, 0);
-    #else
-    view->init(configureAction, refreshAction, 0);
-    #endif
     view->setRootIsDecorated(false);
-    updateGenres(QSet<QString>());
 }
 
 DevicesPage::~DevicesPage()
@@ -144,14 +142,21 @@ void DevicesPage::clear()
 }
 
 QString DevicesPage::activeFsDeviceUdi() const
+{
+    Device *dev=activeFsDevice();
+    return dev ? dev->udi() : QString();
+}
+
+Device * DevicesPage::activeFsDevice() const
 {
     const QModelIndexList selected = view->selectedIndexes();
 
     if (0==selected.size()) {
-        return QString();
+        return 0;
     }
 
     QString udi;
+    Device *activeDev=0;
     foreach (const QModelIndex &idx, selected) {
         QModelIndex index = proxy.mapToSource(idx);
         MusicLibraryItem *item=static_cast<MusicLibraryItem *>(index.internalPointer());
@@ -165,16 +170,33 @@ QString DevicesPage::activeFsDeviceUdi() const
         if (item && MusicLibraryItem::Type_Root==item->itemType()) {
             Device *dev=static_cast<Device *>(item);
             if (Device::Ums!=dev->devType() && Device::RemoteFs!=dev->devType()) {
-                return QString();
+                return 0;
             }
-            if (!udi.isEmpty()) {
-                return QString();
+            if (activeDev) {
+                return 0;
             }
-            udi=dev->udi();
+            activeDev=dev;
         }
     }
 
-    return udi;
+    return activeDev;
+}
+
+QStringList DevicesPage::playableUrls() const
+{
+    QModelIndexList selected = view->selectedIndexes();
+
+    if (0==selected.size()) {
+        return QStringList();
+    }
+    qSort(selected);
+
+    QModelIndexList mapped;
+    foreach (const QModelIndex &idx, selected) {
+        mapped.append(proxy.mapToSource(idx));
+    }
+
+    return DevicesModel::self()->playableUrls(mapped);
 }
 
 QList<Song> DevicesPage::selectedSongs() const
@@ -211,13 +233,27 @@ QList<Song> DevicesPage::selectedSongs() const
     return DevicesModel::self()->songs(mapped);
 }
 
+void DevicesPage::addSelectionToPlaylist(const QString &name, bool replace, quint8 priorty)
+{
+    QStringList files=playableUrls();
+
+    if (!files.isEmpty()) {
+        if (name.isEmpty()) {
+            emit add(files, replace, priorty);
+        } else {
+            emit addSongsToPlaylist(name, files);
+        }
+        view->clearSelection();
+    }
+}
+
 void DevicesPage::itemDoubleClicked(const QModelIndex &)
 {
 //     const QModelIndexList selected = view->selectedIndexes();
 //     if (1!=selected.size()) {
 //         return; //doubleclick should only have one selected item
 //     }
-//     MusicDevicesItem *item = static_cast<MusicDevicesItem *>(proxy.mapToSource(selected.at(0)).internalPointer());
+//     MusicDevicesItem *item = DevicesModel::self()->toItem(proxy.mapToSource(selected.at(0)));
 //     if (MusicDevicesItem::Type_Song==item->itemType()) {
 //         addSelectionToPlaylist();
 //     }
@@ -225,8 +261,9 @@ void DevicesPage::itemDoubleClicked(const QModelIndex &)
 
 void DevicesPage::searchItems()
 {
-    proxy.update(view->searchText().trimmed(), genreCombo->currentIndex()<=0 ? QString() : genreCombo->currentText());
-    if (proxy.enabled()) {
+    QString text=view->searchText().trimmed();
+    proxy.update(text, genreCombo->currentIndex()<=0 ? QString() : genreCombo->currentText());
+    if (proxy.enabled() && !text.isEmpty()) {
         view->expandAll();
     }
 }
@@ -243,6 +280,8 @@ void DevicesPage::controlActions()
     #endif
     bool deviceSelected=false;
     bool busyDevice=false;
+    bool audioCd=false;
+    bool canPlay=false;
     QString udi;
 
     foreach (const QModelIndex &idx, selected) {
@@ -266,11 +305,15 @@ void DevicesPage::controlActions()
             if (!dev->isIdle()) {
                 busyDevice=true;
             }
+            if (Device::AudioCd==dev->devType()) {
+                audioCd=true;
+            }
             #ifdef ENABLE_REMOTE_DEVICES
-            if (Device::RemoteFs==dev->devType()) {
+            else if (Device::RemoteFs==dev->devType()) {
                 remoteDev=true;
             }
             #endif
+            canPlay=dev->canPlaySongs();
             if (udi.isEmpty()) {
                 udi=dev->udi();
             } else if (udi!=dev->udi()) {
@@ -283,20 +326,27 @@ void DevicesPage::controlActions()
         }
     }
 
-    configureAction->setEnabled(!busyDevice && 1==selected.count());
-    refreshAction->setEnabled(!busyDevice && 1==selected.count());
-    copyAction->setEnabled(!busyDevice && haveTracks && !deviceSelected);
-    syncAction->setEnabled(!busyDevice && deviceSelected && connected && 1==selected.count() && singleUdi);
-    mw->deleteSongsAction->setEnabled(!busyDevice && haveTracks && !deviceSelected);
-    mw->editTagsAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
+    DevicesModel::self()->configureAct()->setEnabled(!busyDevice && 1==selected.count() && !audioCd);
+    DevicesModel::self()->refreshAct()->setEnabled(!busyDevice && 1==selected.count());
+    copyAction->setEnabled(!busyDevice && haveTracks && (!deviceSelected || audioCd));
+    syncAction->setEnabled(!audioCd && !busyDevice && deviceSelected && connected && 1==selected.count() && singleUdi &&
+                           MPDConnection::self()->getDetails().dirReadable);
+    StdActions::self()->deleteSongsAction->setEnabled(!audioCd && !busyDevice && haveTracks && !deviceSelected);
+    StdActions::self()->editTagsAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
     #ifdef ENABLE_REPLAYGAIN_SUPPORT
-    mw->replaygainAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
+    StdActions::self()->replaygainAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
     #endif
-    //mw->burnAction->setEnabled(enable && onlyFs);
-    mw->organiseFilesAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
+    StdActions::self()->organiseFilesAction->setEnabled(!busyDevice && haveTracks && onlyFs && singleUdi && !deviceSelected);
+    StdActions::self()->addToPlayQueueAction->setEnabled(canPlay && !selected.isEmpty() && singleUdi && !busyDevice && haveTracks && (audioCd || !deviceSelected));
+    StdActions::self()->addWithPriorityAction->setEnabled(StdActions::self()->addToPlayQueueAction->isEnabled());
+    StdActions::self()->replacePlayQueueAction->setEnabled(StdActions::self()->addToPlayQueueAction->isEnabled());
     #ifdef ENABLE_REMOTE_DEVICES
     forgetDeviceAction->setEnabled(singleUdi && remoteDev);
     #endif
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    DevicesModel::self()->editAct()->setEnabled(!AlbumDetailsDialog::instanceCount() && !busyDevice && 1==selected.count() && audioCd && haveTracks && deviceSelected);
+    #endif
+    menuButton->controlState();
 }
 
 void DevicesPage::copyToLibrary()
@@ -360,24 +410,46 @@ void DevicesPage::refreshDevice()
 
     if (MusicLibraryItem::Type_Root==item->itemType()) {
         Device *dev=static_cast<Device *>(item);
+        QString udi=dev->udi();
         bool full=true;
-        if (dev->childCount() && Device::Mtp!=dev->devType()) {
-            QString udi=dev->udi();
-            switch (MessageBox::questionYesNoCancel(this, i18n("<p>Which type of refresh do you wish to perform?<ul>"
-                                                               "<li>Partial - Only new songs are scanned <i>(quick)</i></li>"
-                                                               "<li>Full - All songs are rescanned <i>(slow)</i></li></ul></p>"),
-                                                    i18n("Refresh"), GuiItem(i18n("Partial")), GuiItem(i18n("Full")))) {
+
+        if (Device::AudioCd==dev->devType()) {
+            // Bit hacky - we use 'full' to determine CDDB/MusicBrainz!
+            #if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
+            switch (MessageBox::questionYesNoCancel(this, i18n("Lookup album and track details?"),
+                                                    i18n("Refresh"), GuiItem(i18n("Via CDDB")), GuiItem(i18n("Via MusicBrainz")))) {
+            case MessageBox::Yes:
+                full=true; // full==true => CDDB
+                break;
+            case MessageBox::No:
+                full=false; // full==false => MusicBrainz
+                break;
+            default:
+                return;
+            }
+            #else
+            if (MessageBox::No==MessageBox::questionYesNo(this, i18n("Lookup album and track details?"), i18n("Refresh"))) {
+                return;
+            }
+            #endif
+            dev=DevicesModel::self()->device(udi);
+        } else {
+            if (dev->childCount() && Device::Mtp!=dev->devType()) {
+                switch (MessageBox::questionYesNoCancel(this, i18n("<p>Which type of refresh do you wish to perform?<ul>"
+                                                                   "<li>Partial - Only new songs are scanned <i>(quick)</i></li>"
+                                                                   "<li>Full - All songs are rescanned <i>(slow)</i></li></ul></p>"),
+                                                        i18n("Refresh"), GuiItem(i18n("Partial")), GuiItem(i18n("Full")))) {
                 case MessageBox::Yes:
                     full=false;
                 case MessageBox::No:
                     break;
                 default:
                     return;
+                }
+                // We have to query for device again, as it could have been unplugged whilst the above question dialog was visible...
+                dev=DevicesModel::self()->device(udi);
             }
-            // We have to query for device again, as it could have been unplugged whilst the above question dialog was visible...
-            dev=DevicesModel::self()->device(udi);
         }
-
         if (dev) {
             dev->rescan(full);
         }
@@ -424,17 +496,22 @@ void DevicesPage::addRemoteDevice()
 {
     #ifdef ENABLE_REMOTE_DEVICES
     RemoteDevicePropertiesDialog *dlg=new RemoteDevicePropertiesDialog(this);
-    dlg->show("cover.jpg", DeviceOptions(), RemoteFsDevice::Details(), DevicePropertiesWidget::Prop_All-DevicePropertiesWidget::Prop_Folder, true);
-    connect(dlg, SIGNAL(updatedSettings(const QString &, const DeviceOptions &, RemoteFsDevice::Details)),
-            DevicesModel::self(), SLOT(addRemoteDevice(const QString &, const DeviceOptions &, RemoteFsDevice::Details)));
+    dlg->show(DeviceOptions(QLatin1String("cover.jpg")), RemoteFsDevice::Details(), DevicePropertiesWidget::Prop_All-DevicePropertiesWidget::Prop_Folder, true);
+    connect(dlg, SIGNAL(updatedSettings(const DeviceOptions &, RemoteFsDevice::Details)),
+            DevicesModel::self(), SLOT(addRemoteDevice(const DeviceOptions &, RemoteFsDevice::Details)));
     #endif
 }
 
 void DevicesPage::forgetRemoteDevice()
 {
     #ifdef ENABLE_REMOTE_DEVICES
-    QString udi=activeFsDeviceUdi();
-    if (!udi.isEmpty() && MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to forget the selected device?"))) {
+    Device *dev=activeFsDevice();
+    if (!dev) {
+        return;
+    }
+    QString udi=dev->udi();
+    QString devName=dev->data();
+    if (MessageBox::Yes==MessageBox::warningYesNo(this, i18n("Are you sure you wish to forget '%1'?").arg(devName))) {
         DevicesModel::self()->removeRemoteDevice(udi);
     }
     #endif
@@ -443,7 +520,6 @@ void DevicesPage::forgetRemoteDevice()
 void DevicesPage::toggleDevice()
 {
     const QModelIndexList selected = view->selectedIndexes();
-
     if (1!=selected.size()) {
         return;
     }
@@ -451,26 +527,33 @@ void DevicesPage::toggleDevice()
     MusicLibraryItem *item=static_cast<MusicLibraryItem *>(proxy.mapToSource(selected.first()).internalPointer());
 
     if (MusicLibraryItem::Type_Root==item->itemType()) {
+        Device *dev=static_cast<Device *>(item);
+        if (dev->isConnected() &&
+            MessageBox::No==MessageBox::warningYesNo(this,
+                                                     Device::AudioCd==dev->devType()
+                                                        ? i18n("Are you sure you wish to eject Audio CD '%1 - %2'?").arg(dev->data()).arg(dev->subText())
+                                                        : i18n("Are you sure you wish to disconnect '%1'?").arg(dev->data()))) {
+            return;
+        }
         static_cast<Device *>(item)->toggle();
     }
 }
 
-#define DIALOG_ERROR MessageBox::error(this, i18n("Action is not currently possible, due to other open dialogs.")); return
-
 void DevicesPage::sync()
 {
     if (0!=SyncDialog::instanceCount()) {
         return;
     }
 
-    if (0!=TagEditor::instanceCount() || 0!=ActionDialog::instanceCount() || 0!=TrackOrganiser::instanceCount()) {
-        DIALOG_ERROR;
+    if (0!=PreferencesDialog::instanceCount() || 0!=TagEditor::instanceCount() || 0!=TrackOrganiser::instanceCount()
+        || 0!=ActionDialog::instanceCount() || 0!=CoverDialog::instanceCount()
+        #ifdef ENABLE_REPLAYGAIN_SUPPORT
+        || 0!=RgDialog::instanceCount()
+        #endif
+        ) {
+        MessageBox::error(this, i18n("Please close other dialogs first."));
+        return;
     }
-    #ifdef ENABLE_REPLAYGAIN_SUPPORT
-    if (0!=RgDialog::instanceCount()) {
-        DIALOG_ERROR;
-    }
-    #endif
 
     const QModelIndexList selected = view->selectedIndexes();
 
@@ -486,35 +569,80 @@ void DevicesPage::sync()
     }
 }
 
-void DevicesPage::updateGenres(const QSet<QString> &g)
+void DevicesPage::updateGenres(const QModelIndex &idx)
 {
-    if (genreCombo->count() && g==genres) {
-        return;
-    }
-
-    genres=g;
-    QStringList entries=g.toList();
-    qSort(entries);
-    entries.prepend(i18n("All Genres"));
-
-    QString currentFilter = genreCombo->currentIndex() ? genreCombo->currentText() : QString();
-
-    genreCombo->clear();
-    genreCombo->addItems(entries);
-    if (0==genres.count()) {
-        genreCombo->setCurrentIndex(0);
-    } else {
-        if (!currentFilter.isEmpty()) {
-            bool found=false;
-            for (int i=1; i<genreCombo->count() && !found; ++i) {
-                if (genreCombo->itemText(i) == currentFilter) {
-                    genreCombo->setCurrentIndex(i);
-                    found=true;
-                }
-            }
-            if (!found) {
-                genreCombo->setCurrentIndex(0);
+    if (idx.isValid()) {
+        QModelIndex m=proxy.mapToSource(idx);
+        if (m.isValid()) {
+            MusicLibraryItem *item=static_cast<MusicLibraryItem *>(m.internalPointer());
+            MusicLibraryItem::Type itemType=item->itemType();
+            if (itemType==MusicLibraryItem::Type_Root) {
+                genreCombo->update(static_cast<MusicLibraryItemRoot *>(item)->genres());
+                return;
+            } else if (itemType==MusicLibraryItem::Type_Artist) {
+                genreCombo->update(static_cast<MusicLibraryItemArtist *>(item)->genres());
+                return;
+            } else if (itemType==MusicLibraryItem::Type_Album) {
+                genreCombo->update(static_cast<MusicLibraryItemAlbum *>(item)->genres());
+                return;
             }
         }
     }
+    genreCombo->update(DevicesModel::self()->genres());
+}
+
+void DevicesPage::updated(const QModelIndex &idx)
+{
+    view->setExpanded(proxy.mapFromSource(idx));
+}
+
+void DevicesPage::cdMatches(const QString &udi, const QList<CdAlbum> &albums)
+{
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    int chosen=0;
+    Device *dev=DevicesModel::self()->device(udi);
+    if (dev && Device::AudioCd==dev->devType()) {
+        if (Device::AudioCd==dev->devType()) {
+            CddbSelectionDialog *dlg=new CddbSelectionDialog(this);
+            chosen=dlg->select(albums);
+            if (chosen<0 || chosen>=albums.count()) {
+                chosen=0;
+            }
+        }
+    }
+
+    // Need to get device again, as it may have been removed!
+    dev=DevicesModel::self()->device(udi);
+    if (dev && Device::AudioCd==dev->devType()) {
+        if (Device::AudioCd==dev->devType()) {
+            static_cast<AudioCdDevice *>(dev)->setDetails(albums.at(chosen));
+        }
+    }
+    #else
+    Q_UNUSED(udi)
+    Q_UNUSED(albums)
+    #endif
+}
+
+void DevicesPage::editDetails()
+{
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    if (AlbumDetailsDialog::instanceCount()) {
+        return;
+    }
+    const QModelIndexList selected = view->selectedIndexes();
+    if (1!=selected.size()) {
+        return;
+    }
+
+    MusicLibraryItem *item=static_cast<MusicLibraryItem *>(proxy.mapToSource(selected.first()).internalPointer());
+
+    if (MusicLibraryItem::Type_Root==item->itemType()) {
+        Device *dev=static_cast<Device *>(item);
+        if (Device::AudioCd==dev->devType()) {
+            AlbumDetailsDialog *dlg=new AlbumDetailsDialog(this);
+            dlg->show(static_cast<AudioCdDevice *>(dev));
+        }
+    }
+    #endif
 }
