@@ -140,6 +140,10 @@ void InfoPage::update(const Song &s, bool force)
         song.revertVariousArtists();
     }
 
+    if (!song.albumartist.isEmpty() && !song.artist.isEmpty() && song.albumartist.length()<song.artist.length() && song.artist.startsWith(song.albumartist)) {
+        song.artist=song.albumartist;
+    }
+
     bool artistChanged=song.artist!=currentSong.artist;
 
     if (!isVisible()) {
