@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,35 +24,33 @@
 #ifndef _HTTP_SOCKET_H_
 #define _HTTP_SOCKET_H_
 
-#include <QtNetwork/QTcpServer>
+#include <QTcpServer>
+
+class QHostAddress;
 
 class HttpSocket : public QTcpServer
 {
     Q_OBJECT
 
 public:
-     HttpSocket(const QString &addr, quint16 p);
-
-     virtual ~HttpSocket() {
-     }
+     HttpSocket(const QString &addr, quint16 p, quint16 prevPort);
+     virtual ~HttpSocket() { }
 
      void terminate();
      void incomingConnection(int socket);
+     QString address() const { return ifaceAddress; }
+     QString configuredAddress() { return cfgAddress; }
 
-     QString address() const {
-         return serverAddress().toString();
-     }
-
-     QString configuredAddress() {
-         return cfgAddr;
-     }
+private:
+     bool openPort(const QHostAddress &a, const QString &addr, quint16 p);
 
 private Q_SLOTS:
      void readClient();
      void discardClient();
 
 private:
-    QString cfgAddr;
+    QString cfgAddress;
+    QString ifaceAddress;
     bool terminated;
 };
 
