@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -21,11 +21,11 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include <QtCore/QModelIndex>
-#include <QtCore/QDataStream>
-#include <QtCore/QMimeData>
-#include <QtCore/QStringList>
-#include <QtGui/QPainter>
+#include <QModelIndex>
+#include <QDataStream>
+#include <QMimeData>
+#include <QStringList>
+#include <QPainter>
 #include "localize.h"
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobal>
@@ -117,7 +117,7 @@ void AlbumsModel::setCoverSize(MusicLibraryItemAlbum::CoverSize size)
 }
 
 AlbumsModel::AlbumsModel(QObject *parent)
-    : QAbstractItemModel(parent)
+    : ActionModel(parent)
     , enabled(false)
 //     , coversRequested(false)
 {
@@ -203,6 +203,8 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
         AlbumItem *al=static_cast<AlbumItem *>(item);
 
         switch (role) {
+        default:
+            return ActionModel::data(index, role);
         case ItemView::Role_Image:
         case Qt::DecorationRole: {
             if (al->cover) {
@@ -221,10 +223,13 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
                     cSize=stdSize=22;
                 }
                 theDefaultIcon = new QPixmap(Icons::albumIcon.pixmap(stdSize, stdSize)
-                                            .scaled(QSize(cSize, cSize), Qt::KeepAspectRatio, Qt::SmoothTransformation));
+                                            .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
             }
             if (!al->coverRequested && iSize && Song::SingleTracks!=al->type) {
-                al->getCover(true);
+                al->getCover();
+                if (al->cover) {
+                    return *(al->cover);
+                }
                 al->coverRequested=true;
             }
             return *theDefaultIcon;
@@ -260,6 +265,8 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
         SongItem *si=static_cast<SongItem *>(item);
 
         switch (role) {
+        default:
+            return ActionModel::data(index, role);
         case Qt::DecorationRole:
             return Icon(Song::Playlist==si->type ? "view-media-playlist" : "audio-x-generic");
         case Qt::ToolTipRole: {
@@ -443,10 +450,10 @@ void AlbumsModel::update(const MusicLibraryItemRoot *root)
     }
 }
 
-void AlbumsModel::setCover(const Song &song, const QImage &img, const QString &file)
+void AlbumsModel::setCover(const Song &song, const QImage &img, const QString &file, bool update)
 {
     Q_UNUSED(file)
-    if (img.isNull()) {
+    if (img.isNull() || song.isArtistImageRequest()) {
         return;
     }
 
@@ -455,14 +462,21 @@ void AlbumsModel::setCover(const Song &song, const QImage &img, const QString &f
 
     for (int row=0; it!=end; ++it, ++row) {
         if ((*it)->artist==song.albumArtist() && (*it)->album==song.album) {
-            (*it)->cover=new QPixmap(QPixmap::fromImage(img.scaled(QSize(iconSize(), iconSize()), Qt::KeepAspectRatio, Qt::SmoothTransformation)));
-            QModelIndex idx=index(row, 0, QModelIndex());
-            emit dataChanged(idx, idx);
+            if (!(*it)->cover || update) {
+                (*it)->cover=new QPixmap(QPixmap::fromImage(img.scaled(QSize(iconSize(), iconSize()), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+                QModelIndex idx=index(row, 0, QModelIndex());
+                emit dataChanged(idx, idx);
+            }
             return;
         }
     }
 }
 
+void AlbumsModel::updateCover(const Song &song, const QImage &img, const QString &file)
+{
+    setCover(song, img, file, true);
+}
+
 void AlbumsModel::clear()
 {
     beginResetModel();
@@ -482,12 +496,16 @@ void AlbumsModel::setEnabled(bool e)
     if (enabled) {
         connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
                 this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+        connect(Covers::self(), SIGNAL(coverUpdated(const Song &, const QImage &, const QString &)),
+                this, SLOT(updateCover(const Song &, const QImage &, const QString &)));
 //         connect(MusicLibraryModel::self(), SIGNAL(updated(const MusicLibraryItemRoot *)), AlbumsModel::self(), SLOT(update(const MusicLibraryItemRoot *)));
         update(MusicLibraryModel::self()->root());
     } else {
         clear();
         disconnect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
                     this, SLOT(setCover(const Song&, const QImage &, const QString &)));
+        disconnect(Covers::self(), SIGNAL(coverUpdated(const Song &, const QImage &, const QString &)),
+                   this, SLOT(updateCover(const Song &, const QImage &, const QString &)));
 //         disconnect(MusicLibraryModel::self(), SIGNAL(updated(const MusicLibraryItemRoot *)), AlbumsModel::self(), SLOT(update(const MusicLibraryItemRoot *)));
     }
 }
@@ -572,7 +590,7 @@ quint32 AlbumsModel::AlbumItem::totalTime()
     return time;
 }
 
-void AlbumsModel::AlbumItem::getCover(bool urgent)
+void AlbumsModel::AlbumItem::getCover()
 {
     if (Song::SingleTracks!=type && songs.count()) {
         SongItem *firstSong=songs.first();
@@ -585,7 +603,10 @@ void AlbumsModel::AlbumItem::getCover(bool urgent)
         s.album=album;
         s.year=year;
         s.file=firstSong->file;
-        Covers::self()->requestCover(s, urgent);
+        Covers::Image img=Covers::self()->requestImage(s);
+        if (!img.img.isNull()) {
+            cover=new QPixmap(QPixmap::fromImage(img.img.scaled(QSize(iconSize(), iconSize()), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+        }
     }
 }
 
