@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -32,9 +32,10 @@
 #include "mpdconnection.h"
 #include "song.h"
 #include "localize.h"
-#include <QtXml/QXmlStreamReader>
-#include <QtXml/QXmlStreamWriter>
-#include <QtCore/QFile>
+#include "qtiocompressor/qtiocompressor.h"
+#include <QXmlStreamReader>
+#include <QXmlStreamWriter>
+#include <QFile>
 
 MusicLibraryItemArtist * MusicLibraryItemRoot::artist(const Song &s, bool create)
 {
@@ -58,7 +59,7 @@ MusicLibraryItemArtist * MusicLibraryItemRoot::createArtist(const Song &s)
 
 void MusicLibraryItemRoot::groupSingleTracks()
 {
-    if (!supportsAlbumArtist) {
+    if (!supportsAlbumArtist || isFlat) {
         return;
     }
 
@@ -101,7 +102,7 @@ void MusicLibraryItemRoot::groupSingleTracks()
 
 void MusicLibraryItemRoot::groupMultipleArtists()
 {
-    if (!supportsAlbumArtist) {
+    if (!supportsAlbumArtist || isFlat) {
         return;
     }
 
@@ -161,7 +162,7 @@ void MusicLibraryItemRoot::groupMultipleArtists()
 
 bool MusicLibraryItemRoot::isFromSingleTracks(const Song &s) const
 {
-    if (supportsAlbumArtist && !s.file.isEmpty()) {
+    if (!isFlat && (supportsAlbumArtist && !s.file.isEmpty())) {
         QHash<QString, int>::ConstIterator it=m_indexes.find(i18n("Various Artists"));
 
         if (m_indexes.end()!=it) {
@@ -173,6 +174,9 @@ bool MusicLibraryItemRoot::isFromSingleTracks(const Song &s) const
 
 void MusicLibraryItemRoot::refreshIndexes()
 {
+    if (isFlat) {
+        return;
+    }
     m_indexes.clear();
     int i=0;
     foreach (MusicLibraryItem *item, m_childItems) {
@@ -182,6 +186,10 @@ void MusicLibraryItemRoot::refreshIndexes()
 
 void MusicLibraryItemRoot::remove(MusicLibraryItemArtist *artist)
 {
+    if (isFlat) {
+        return;
+    }
+
     int index=m_childItems.indexOf(artist);
 
     if (index<0 || index>=m_childItems.count()) {
@@ -204,15 +212,25 @@ QSet<Song> MusicLibraryItemRoot::allSongs(bool revertVa) const
 {
     QSet<Song> songs;
 
-    foreach (const MusicLibraryItem *artist, m_childItems) {
-        foreach (const MusicLibraryItem *album, static_cast<const MusicLibraryItemContainer *>(artist)->childItems()) {
-            foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
-                if (revertVa) {
-                    Song s=static_cast<const MusicLibraryItemSong *>(song)->song();
-                    s.revertVariousArtists();
-                    songs.insert(s);
-                } else {
-                    songs.insert(static_cast<const MusicLibraryItemSong *>(song)->song());
+    foreach (const MusicLibraryItem *child, m_childItems) {
+        if (MusicLibraryItem::Type_Song==child->itemType()) {
+            if (revertVa) {
+                Song s=static_cast<const MusicLibraryItemSong *>(child)->song();
+                s.revertVariousArtists();
+                songs.insert(s);
+            } else {
+                songs.insert(static_cast<const MusicLibraryItemSong *>(child)->song());
+            }
+        } else {
+            foreach (const MusicLibraryItem *album, static_cast<const MusicLibraryItemContainer *>(child)->childItems()) {
+                foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
+                    if (revertVa) {
+                        Song s=static_cast<const MusicLibraryItemSong *>(song)->song();
+                        s.revertVariousArtists();
+                        songs.insert(s);
+                    } else {
+                        songs.insert(static_cast<const MusicLibraryItemSong *>(song)->song());
+                    }
                 }
             }
         }
@@ -222,15 +240,25 @@ QSet<Song> MusicLibraryItemRoot::allSongs(bool revertVa) const
 
 void MusicLibraryItemRoot::getDetails(QSet<QString> &artists, QSet<QString> &albumArtists, QSet<QString> &albums, QSet<QString> &genres)
 {
-    foreach (const MusicLibraryItem *artist, m_childItems) {
-        foreach (const MusicLibraryItem *album, static_cast<const MusicLibraryItemContainer *>(artist)->childItems()) {
-            foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
-                const Song &s=static_cast<const MusicLibraryItemSong *>(song)->song();
-                artists.insert(s.artist);
-                albumArtists.insert(s.albumArtist());
-                albums.insert(s.album);
-                if (!s.genre.isEmpty()) {
-                    genres.insert(s.genre);
+    foreach (const MusicLibraryItem *child, m_childItems) {
+        if (MusicLibraryItem::Type_Song==child->itemType()) {
+            const Song &s=static_cast<const MusicLibraryItemSong *>(child)->song();
+            artists.insert(s.artist);
+            albumArtists.insert(s.albumArtist());
+            albums.insert(s.album);
+            if (!s.genre.isEmpty()) {
+                genres.insert(s.genre);
+            }
+        } else {
+            foreach (const MusicLibraryItem *album, static_cast<const MusicLibraryItemContainer *>(child)->childItems()) {
+                foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
+                    const Song &s=static_cast<const MusicLibraryItemSong *>(song)->song();
+                    artists.insert(s.artist);
+                    albumArtists.insert(s.albumArtist());
+                    albums.insert(s.album);
+                    if (!s.genre.isEmpty()) {
+                        genres.insert(s.genre);
+                    }
                 }
             }
         }
@@ -239,6 +267,10 @@ void MusicLibraryItemRoot::getDetails(QSet<QString> &artists, QSet<QString> &alb
 
 void MusicLibraryItemRoot::updateSongFile(const Song &from, const Song &to)
 {
+    if (isFlat) {
+        return;
+    }
+
     MusicLibraryItemArtist *art=artist(from, false);
     if (art) {
         MusicLibraryItemAlbum *alb=art->album(from, false);
@@ -256,23 +288,42 @@ void MusicLibraryItemRoot::updateSongFile(const Song &from, const Song &to)
 static quint32 constVersion=8;
 static QLatin1String constTopTag("CantataLibrary");
 
-void MusicLibraryItemRoot::toXML(const QString &filename, const QDateTime &date) const
+void MusicLibraryItemRoot::toXML(const QString &filename, const QDateTime &date, MusicLibraryProgressMonitor *prog) const
 {
-    QFile file(filename);
-    if (!file.open(QIODevice::WriteOnly)) {
+    if (isFlat) {
         return;
     }
 
-    //Write the header info
-    QXmlStreamWriter writer(&file);
-    toXML(writer, date);
-    file.close();
+    // If saving device cache, and we have NO items, then remove cache file...
+    if (0==childCount() && date==QDateTime()) {
+        if (QFile::exists(filename)) {
+            QFile::remove(filename);
+        }
+        return;
+    }
+
+    QFile file(filename);
+    QtIOCompressor compressor(&file);
+    compressor.setStreamFormat(QtIOCompressor::GzipFormat);
+    if (!compressor.open(QIODevice::WriteOnly)) {
+        return;
+    }
+
+    QXmlStreamWriter writer(&compressor);
+    toXML(writer, date, prog);
+    compressor.close();
 }
 
-void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date) const
+void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date, MusicLibraryProgressMonitor *prog) const
 {
-    writer.setAutoFormatting(true);
+    if (isFlat) {
+        return;
+    }
+
+    quint64 total=0;
+    quint64 count=0;
     writer.writeStartDocument();
+    QString unknown=i18n("Unknown");
 
     //Start with the document
     writer.writeStartElement(constTopTag);
@@ -280,18 +331,38 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
     writer.writeAttribute("date", QString::number(date.toTime_t()));
     writer.writeAttribute("groupSingle", MPDParseUtils::groupSingle() ? "true" : "false");
     writer.writeAttribute("groupMultiple", MPDParseUtils::groupMultiple() ? "true" : "false");
+
+    foreach (const MusicLibraryItem *a, childItems()) {
+        foreach (const MusicLibraryItem *al, static_cast<const MusicLibraryItemArtist *>(a)->childItems()) {
+            total+=al->childCount();
+            if (prog && prog->wasStopped()) {
+                return;
+            }
+        }
+    }
+
+    writer.writeAttribute("numTracks", QString::number(total));
+    if (prog) {
+        prog->writeProgress(0.0);
+    }
     //Loop over all artist, albums and tracks.
     foreach (const MusicLibraryItem *a, childItems()) {
         const MusicLibraryItemArtist *artist = static_cast<const MusicLibraryItemArtist *>(a);
         writer.writeStartElement("Artist");
         writer.writeAttribute("name", artist->data());
+        if (!artist->imageUrl().isEmpty()) {
+            writer.writeAttribute("img", artist->imageUrl());
+        }
         foreach (const MusicLibraryItem *al, artist->childItems()) {
+            if (prog && prog->wasStopped()) {
+                return;
+            }
             const MusicLibraryItemAlbum *album = static_cast<const MusicLibraryItemAlbum *>(al);
             QString albumGenre=!album->childItems().isEmpty() ? static_cast<const MusicLibraryItemSong *>(album->childItems().at(0))->song().genre : QString();
             writer.writeStartElement("Album");
             writer.writeAttribute("name", album->data());
             writer.writeAttribute("year", QString::number(album->year()));
-            if (!albumGenre.isEmpty()) {
+            if (!albumGenre.isEmpty() && albumGenre!=unknown) {
                 writer.writeAttribute("genre", albumGenre);
             }
             if (album->isSingleTracks()) {
@@ -299,6 +370,9 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
             } else if (album->isMultipleArtists()) {
                 writer.writeAttribute("multipleArtists", "true");
             }
+            if (!album->imageUrl().isEmpty()) {
+                writer.writeAttribute("img", album->imageUrl());
+            }
             foreach (const MusicLibraryItem *t, album->childItems()) {
                 const MusicLibraryItemSong *track = static_cast<const MusicLibraryItemSong *>(t);
                 bool wroteArtist=false;
@@ -321,11 +395,11 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
                     writer.writeAttribute("artist", track->song().artist);
                     wroteArtist=true;
                 }
-                if (!track->song().albumartist.isEmpty() && track->song().albumartist!=artist->data()) {
+                if (track->song().albumartist!=artist->data()) {
                     writer.writeAttribute("albumartist", track->song().albumartist);
                 }
 //                 writer.writeAttribute("id", QString::number(track->song().id));
-                if (!track->song().genre.isEmpty() && track->song().genre!=albumGenre) {
+                if (!track->song().genre.isEmpty() && track->song().genre!=albumGenre && track->song().genre!=unknown) {
                     writer.writeAttribute("genre", track->song().genre);
                 }
                 if (album->isSingleTracks()) {
@@ -339,6 +413,13 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
                 if (track->song().year != album->year()) {
                     writer.writeAttribute("year", QString::number(track->song().year));
                 }
+                if (track->song().guessed) {
+                    writer.writeAttribute("guessed", "true");
+                }
+                if (prog && !prog->wasStopped() && total>0) {
+                    count++;
+                    prog->writeProgress((count*100.0)/(total*1.0));
+                }
             }
             writer.writeEndElement();
         }
@@ -349,33 +430,48 @@ void MusicLibraryItemRoot::toXML(QXmlStreamWriter &writer, const QDateTime &date
     writer.writeEndDocument();
 }
 
-quint32 MusicLibraryItemRoot::fromXML(const QString &filename, const QDateTime &date, const QString &baseFolder)
+quint32 MusicLibraryItemRoot::fromXML(const QString &filename, const QDateTime &date, const QString &baseFolder, MusicLibraryProgressMonitor *prog)
 {
-    QFile file(filename);
-    if (!file.open(QIODevice::ReadOnly)) {
+    if (isFlat) {
         return 0;
     }
 
-    QXmlStreamReader reader(&file);
-    quint32 rv=fromXML(reader, date, baseFolder);
-    file.close();
+    QFile file(filename);
+    QtIOCompressor compressor(&file);
+    compressor.setStreamFormat(QtIOCompressor::GzipFormat);
+    if (!compressor.open(QIODevice::ReadOnly)) {
+        return 0;
+    }
+
+    QXmlStreamReader reader(&compressor);
+    quint32 rv=fromXML(reader, date, baseFolder, prog);
+    compressor.close();
     return rv;
 }
 
-quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime &date, const QString &baseFolder)
+quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime &date, const QString &baseFolder, MusicLibraryProgressMonitor *prog)
 {
+    if (isFlat) {
+        return 0;
+    }
+
     MusicLibraryItemArtist *artistItem = 0;
     MusicLibraryItemAlbum *albumItem = 0;
-    MusicLibraryItemSong *songItem = 0;
     Song song;
     quint32 xmlDate=0;
+    QString unknown=i18n("Unknown");
+    quint64 total=0;
+    quint64 count=0;
+    bool gs=MPDParseUtils::groupSingle();
+    bool gm=MPDParseUtils::groupMultiple();
 
-    while (!reader.atEnd()) {
+    if (prog) {
+        prog->readProgress(0.0);
+    }
+
+    while (!reader.atEnd() && (!prog || !prog->wasStopped())) {
         reader.readNext();
 
-        /**
-         * TODO: CHECK FOR ERRORS
-         */
         if (!reader.error() && reader.isStartElement()) {
             QString element = reader.name().toString();
             QXmlStreamAttributes attributes=reader.attributes();
@@ -383,16 +479,23 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
             if (constTopTag == element) {
                 quint32 version = attributes.value("version").toString().toUInt();
                 xmlDate = attributes.value("date").toString().toUInt();
-                bool gs = QLatin1String("true")==attributes.value("groupSingle").toString();
-                bool gm = QLatin1String("true")==attributes.value("groupMultiple").toString();
+                gs = QLatin1String("true")==attributes.value("groupSingle").toString();
+                gm = QLatin1String("true")==attributes.value("groupMultiple").toString();
 
-                if ( version < constVersion || (date.isValid() && xmlDate < date.toTime_t()) || gs!=MPDParseUtils::groupSingle() || gm!=MPDParseUtils::groupMultiple()) {
+                if ( version < constVersion || (date.isValid() && xmlDate < date.toTime_t())) {
                     return 0;
                 }
+                if (prog) {
+                    total=attributes.value("numTracks").toString().toUInt();
+                }
             } else if (QLatin1String("Artist")==element) {
                 song.type=Song::Standard;
                 song.artist=song.albumartist=attributes.value("name").toString();
                 artistItem = createArtist(song);
+                QString img = attributes.value("img").toString();
+                if (!img.isEmpty()) {
+                    artistItem->setImageUrl(img);
+                }
             } else if (QLatin1String("Album")==element) {
                 song.album=attributes.value("name").toString();
                 song.year=attributes.value("year").toString().toUInt();
@@ -401,6 +504,10 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                     song.file.append("dummy.mp3");
                 }
                 albumItem = artistItem->createAlbum(song);
+                QString img = attributes.value("img").toString();
+                if (!img.isEmpty()) {
+                    albumItem->setImageUrl(img);
+                }
                 if (QLatin1String("true")==attributes.value("singleTracks").toString()) {
                     albumItem->setIsSingleTracks();
                     song.type=Song::SingleTracks;
@@ -415,8 +522,7 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                 song.file=attributes.value("file").toString();
                 if (QLatin1String("true")==attributes.value("playlist").toString()) {
                     song.type=Song::Playlist;
-                    songItem = new MusicLibraryItemSong(song, albumItem);
-                    albumItem->append(songItem);
+                    albumItem->append(new MusicLibraryItemSong(song, albumItem));
                     song.type=Song::Standard;
                 } else {
                     if (!baseFolder.isEmpty() && song.file.startsWith(baseFolder)) {
@@ -424,6 +530,8 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                     }
                     if (attributes.hasAttribute("genre")) {
                         song.genre=attributes.value("genre").toString();
+                    } else if (song.genre.isEmpty()) {
+                        song.genre=unknown;
                     }
                     if (attributes.hasAttribute("artist")) {
                         song.artist=attributes.value("artist").toString();
@@ -468,21 +576,45 @@ quint32 MusicLibraryItemRoot::fromXML(QXmlStreamReader &reader, const QDateTime
                     }
 
                     song.fillEmptyFields();
-                    songItem = new MusicLibraryItemSong(song, albumItem);
-                    albumItem->append(songItem);
+                    if (QLatin1String("true")==attributes.value("guessed").toString()) {
+                        song.guessed=true;
+                    }
+                    albumItem->append(new MusicLibraryItemSong(song, albumItem));
                     albumItem->addGenre(song.genre);
                     artistItem->addGenre(song.genre);
                     addGenre(song.genre);
+
+                    if (prog && !prog->wasStopped() && total>0) {
+                        count++;
+                        prog->readProgress((count*100.0)/(total*1.0));
+                    }
                 }
             }
         }
     }
 
+    // Grouping has changed!
+    if (gs!=MPDParseUtils::groupSingle() || gm!=MPDParseUtils::groupMultiple()) {
+        if (gs && gm) {
+            // Now using both groupings, where previously we were using none.
+            // ...so, jsut apply grouping!
+            groupSingleTracks();
+            groupMultipleArtists();
+        } else {
+            // Mixed groping, so need to redo from scratch...
+            toggleGrouping();
+        }
+    }
+
     return xmlDate;
 }
 
 void MusicLibraryItemRoot::add(const QSet<Song> &songs)
 {
+    if (isFlat) {
+        return;
+    }
+
     MusicLibraryItemArtist *artistItem = 0;
     MusicLibraryItemAlbum *albumItem = 0;
 
@@ -498,8 +630,7 @@ void MusicLibraryItemRoot::add(const QSet<Song> &songs)
             albumItem = artistItem->album(s);
         }
 
-        MusicLibraryItemSong *songItem = new MusicLibraryItemSong(s, albumItem);
-        albumItem->append(songItem);
+        albumItem->append(new MusicLibraryItemSong(s, albumItem));
         albumItem->addGenre(s.genre);
         artistItem->addGenre(s.genre);
         addGenre(s.genre);
@@ -508,12 +639,15 @@ void MusicLibraryItemRoot::add(const QSet<Song> &songs)
 
 void MusicLibraryItemRoot::toggleGrouping()
 {
+    if (isFlat) {
+        return;
+    }
+
     // Grouping has changed, so we need to recreate whole structure from list of songs.
     QSet<Song> songs=allSongs();
     clearItems();
     MusicLibraryItemArtist *artistItem = 0;
     MusicLibraryItemAlbum *albumItem = 0;
-    MusicLibraryItemSong *songItem = 0;
 
     foreach (Song currentSong, songs) {
         if (Song::Standard!=currentSong.type && Song::Playlist!=currentSong.type) {
@@ -527,8 +661,7 @@ void MusicLibraryItemRoot::toggleGrouping()
             albumItem = artistItem->album(currentSong);
         }
 
-        songItem = new MusicLibraryItemSong(currentSong, albumItem);
-        albumItem->append(songItem);
+        albumItem->append(new MusicLibraryItemSong(currentSong, albumItem));
         albumItem->addGenre(currentSong.genre);
         artistItem->addGenre(currentSong.genre);
         addGenre(currentSong.genre);
@@ -545,15 +678,15 @@ void MusicLibraryItemRoot::toggleGrouping()
 
 void MusicLibraryItemRoot::clearItems()
 {
+    qDeleteAll(m_childItems);
     m_childItems.clear();
     m_indexes.clear();
     m_genres.clear();
-    qDeleteAll(m_childItems);
 }
 
-QString MusicLibraryItemRoot::songArtist(const Song &s)
+QString MusicLibraryItemRoot::songArtist(const Song &s) const
 {
-    if (!supportsAlbumArtist) {
+    if (isFlat || !supportsAlbumArtist) {
         return s.artist;
     }
 
