@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -40,14 +40,16 @@
 #include "localize.h"
 #include "utils.h"
 #include "icons.h"
-#include <QtGui/QCommonStyle>
-#include <QtCore/QFile>
-#include <QtCore/QTimer>
-#include <QtCore/QStringRef>
-#include <QtCore/QDateTime>
-#include <QtCore/QDir>
-#include <QtCore/QMimeData>
-#include <QtCore/QStringList>
+#include "stdactions.h"
+#include "qtiocompressor/qtiocompressor.h"
+#include <QCommonStyle>
+#include <QFile>
+#include <QTimer>
+#include <QStringRef>
+#include <QDateTime>
+#include <QDir>
+#include <QMimeData>
+#include <QStringList>
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobal>
 #endif
@@ -69,14 +71,38 @@ MusicLibraryModel * MusicLibraryModel::self()
     #endif
 }
 
-static const QLatin1String constLibraryCache("library/");
-static const QLatin1String constLibraryExt(".xml");
+const QLatin1String MusicLibraryModel::constLibraryCache("library/");
+const QLatin1String MusicLibraryModel::constLibraryExt(".xml");
+const QLatin1String MusicLibraryModel::constLibraryCompressedExt(".xml.gz");
 
 static const QString cacheFileName()
 {
-    QString fileName=MPDConnection::self()->getDetails().hostname+constLibraryExt;
+    QString fileName=MPDConnection::self()->getDetails().hostname+MusicLibraryModel::constLibraryCompressedExt;
     fileName.replace('/', '_');
-    return Utils::cacheDir(constLibraryCache)+fileName;
+    return Utils::cacheDir(MusicLibraryModel::constLibraryCache)+fileName;
+}
+
+void MusicLibraryModel::convertCache(const QString &compressedName)
+{
+    QString prev=compressedName;
+    prev.replace(constLibraryCompressedExt, constLibraryExt);
+
+    if (QFile::exists(prev) && !QFile::exists(compressedName)) {
+        QFile old(prev);
+        if (old.open(QIODevice::ReadOnly)) {
+            QByteArray a=old.readAll();
+            old.close();
+
+            QFile newCache(compressedName);
+            QtIOCompressor compressor(&newCache);
+            compressor.setStreamFormat(QtIOCompressor::GzipFormat);
+            if (compressor.open(QIODevice::WriteOnly)) {
+                compressor.write(a);
+                compressor.close();
+                QFile::remove(prev);
+            }
+        }
+    }
 }
 
 void MusicLibraryModel::cleanCache()
@@ -98,14 +124,21 @@ void MusicLibraryModel::cleanCache()
     }
 }
 
-MusicLibraryModel::MusicLibraryModel(QObject *parent)
-    : QAbstractItemModel(parent)
+MusicLibraryModel::MusicLibraryModel(QObject *parent, bool isMpdModel)
+    : ActionModel(parent)
     , rootItem(new MusicLibraryItemRoot)
 {
-    connect(Covers::self(), SIGNAL(artistImage(const QString &, const QImage &)),
-            this, SLOT(setArtistImage(const QString &, const QImage &)));
-    connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
-            this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+    if (isMpdModel)
+    {
+        connect(Covers::self(), SIGNAL(artistImage(const Song &, const QImage &, const QString &)),
+                this, SLOT(setArtistImage(const Song &, const QImage &)));
+        connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
+                this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+        connect(Covers::self(), SIGNAL(coverUpdated(const Song &, const QImage &, const QString &)),
+                this, SLOT(updateCover(const Song &, const QImage &, const QString &)));
+        connect(MPDConnection::self(), SIGNAL(musicLibraryUpdated(MusicLibraryItemRoot *, QDateTime)),
+                this, SLOT(updateMusicLibrary(MusicLibraryItemRoot *, QDateTime)));
+    }
 }
 
 MusicLibraryModel::~MusicLibraryModel()
@@ -302,7 +335,7 @@ QVariant MusicLibraryModel::data(const QModelIndex &index, int role) const
             return MusicLibraryItemAlbum::itemSize();
         }
     default:
-        return QVariant();
+        return ActionModel::data(index, role);
     }
     return QVariant();
 }
@@ -331,7 +364,7 @@ QModelIndex MusicLibraryModel::findSongIndex(const Song &s) const
         MusicLibraryItemAlbum *albumItem = artistItem->album(s, false);
         if (albumItem) {
             foreach (MusicLibraryItem *songItem, albumItem->childItems()) {
-                if (songItem->data()==s.title) {
+                if (songItem->data()==s.displayTitle()) {
                     return createIndex(albumItem->childItems().indexOf(songItem), 0, songItem);
                 }
             }
@@ -341,6 +374,14 @@ QModelIndex MusicLibraryModel::findSongIndex(const Song &s) const
     return QModelIndex();
 }
 
+QModelIndex MusicLibraryModel::findArtistsIndex(const QString &artist) const
+{
+    Song s;
+    s.artist=artist;
+    MusicLibraryItemArtist *artistItem = rootItem->artist(s, false);
+    return artistItem ? index(rootItem->childItems().indexOf(artistItem), 0, QModelIndex()) : QModelIndex();
+}
+
 const MusicLibraryItem * MusicLibraryModel::findSong(const Song &s) const
 {
     MusicLibraryItemArtist *artistItem = rootItem->artist(s, false);
@@ -348,7 +389,7 @@ const MusicLibraryItem * MusicLibraryModel::findSong(const Song &s) const
         MusicLibraryItemAlbum *albumItem = artistItem->album(s, false);
         if (albumItem) {
             foreach (const MusicLibraryItem *songItem, albumItem->childItems()) {
-                if (songItem->data()==s.title) {
+                if (songItem->data()==s.displayTitle() && static_cast<const MusicLibraryItemSong *>(songItem)->song().track==s.track) {
                     return songItem;
                 }
             }
@@ -409,6 +450,19 @@ bool MusicLibraryModel::updateSong(const Song &orig, const Song &edit)
         foreach (MusicLibraryItem *song, albumItem->childItems()) {
             if (static_cast<MusicLibraryItemSong *>(song)->song()==orig) {
                 static_cast<MusicLibraryItemSong *>(song)->setSong(edit);
+                if (orig.genre!=edit.genre) {
+                    albumItem->updateGenres();
+                    artistItem->updateGenres();
+                    rootItem->updateGenres();
+                }
+
+                if (orig.year!=edit.year) {
+                    if (albumItem->updateYear()) {
+                        QModelIndex idx=index(artistItem->childItems().indexOf(albumItem), 0, index(rootItem->childItems().indexOf(artistItem), 0, QModelIndex()));
+                        emit dataChanged(idx, idx);
+                    }
+                }
+
                 QModelIndex idx=index(songRow, 0, index(artistItem->childItems().indexOf(albumItem), 0, index(rootItem->childItems().indexOf(artistItem), 0, QModelIndex())));
                 emit dataChanged(idx, idx);
                 return true;
@@ -437,7 +491,7 @@ void MusicLibraryModel::addSongToList(const Song &s)
     quint32 year=albumItem->year();
     foreach (const MusicLibraryItem *songItem, albumItem->childItems()) {
         const MusicLibraryItemSong *song=static_cast<const MusicLibraryItemSong *>(songItem);
-        if (song->track()==s.track && song->disc()==s.disc && song->data()==s.title) {
+        if (song->track()==s.track && song->disc()==s.disc && song->data()==s.displayTitle()) {
             return;
         }
     }
@@ -513,11 +567,18 @@ void MusicLibraryModel::updateSongFile(const Song &from, const Song &to)
 
 void MusicLibraryModel::removeCache()
 {
-    //Check if dir exists
     QString cacheFile(cacheFileName());
     if (QFile::exists(cacheFile)) {
         QFile::remove(cacheFile);
     }
+
+    // Remove old (non-compressed) cache file as well...
+    QString oldCache=cacheFile;
+    oldCache.replace(constLibraryCompressedExt, constLibraryExt);
+    if (oldCache!=cacheFile && QFile::exists(oldCache)) {
+        QFile::remove(oldCache);
+    }
+
     databaseTime = QDateTime();
 }
 
@@ -526,6 +587,15 @@ void MusicLibraryModel::getDetails(QSet<QString> &artists, QSet<QString> &albumA
     rootItem->getDetails(artists, albumArtists, albums, genres);
 }
 
+QSet<QString> MusicLibraryModel::getAlbumArtists()
+{
+    QSet<QString> a;
+    foreach (MusicLibraryItem *i, rootItem->childItems()) {
+        a.insert(i->data());
+    }
+    return a;
+}
+
 void MusicLibraryModel::updateMusicLibrary(MusicLibraryItemRoot *newroot, QDateTime dbUpdate, bool fromFile)
 {
     if (databaseTime >= dbUpdate) {
@@ -565,11 +635,10 @@ void MusicLibraryModel::updateMusicLibrary(MusicLibraryItemRoot *newroot, QDateT
         if (!fromFile && (needToSave || needToUpdate)) {
             toXML(rootItem, dbUpdate);
         }
-
-        AlbumsModel::self()->update(rootItem);
-//         emit updated(rootItem);
-        emit updateGenres(rootItem->genres());
     }
+
+    AlbumsModel::self()->update(rootItem);
+    emit updateGenres(rootItem->genres());
 }
 
 bool MusicLibraryModel::update(const QSet<Song> &songs)
@@ -596,54 +665,50 @@ void MusicLibraryModel::toggleGrouping()
     endResetModel();
 }
 
-void MusicLibraryModel::setArtistImage(const QString &artist, const QImage &img)
+void MusicLibraryModel::setArtistImage(const Song &song, const QImage &img, bool update)
 {
-    if (!rootItem->useArtistImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize()) {
+    if (!rootItem->useArtistImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize() ||
+        song.file.startsWith("http://") || song.name.startsWith("http://")) {
         return;
     }
 
-    Song song;
-    song.artist=song.albumartist=artist;
     MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
-    if (artistItem && static_cast<const MusicLibraryItemArtist *>(artistItem)->setCover(img)) {
+    if (artistItem && artistItem->setCover(img, update)) {
         QModelIndex idx=index(rootItem->childItems().indexOf(artistItem), 0, QModelIndex());
         emit dataChanged(idx, idx);
     }
 }
 
 void MusicLibraryModel::setCover(const Song &song, const QImage &img, const QString &file)
+{
+    setCover(song, img, file, false);
+}
+
+void MusicLibraryModel::updateCover(const Song &song, const QImage &img, const QString &file)
+{
+    if (song.isArtistImageRequest()) {
+        setArtistImage(song, img, true);
+    } else {
+        setCover(song, img, file, true);
+    }
+}
+
+void MusicLibraryModel::setCover(const Song &song, const QImage &img, const QString &file, bool update)
 {
     Q_UNUSED(file)
-    if (!rootItem->useAlbumImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize()) {
+    if (!rootItem->useAlbumImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize() ||
+        song.file.startsWith("http:/") || song.name.startsWith("http:/")) {
         return;
     }
 
     MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
     if (artistItem) {
         MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
-        if (albumItem && static_cast<const MusicLibraryItemAlbum *>(albumItem)->setCover(img)) {
+        if (albumItem && albumItem->setCover(img, update)) {
             QModelIndex idx=index(artistItem->childItems().indexOf(albumItem), 0, index(rootItem->childItems().indexOf(artistItem), 0, QModelIndex()));
             emit dataChanged(idx, idx);
         }
     }
-
-//     int i=0;
-//     foreach (const MusicLibraryItem *artistItem, rootItem->childItems()) {
-//         if (artistItem->data()==artist) {
-//             int j=0;
-//             foreach (const MusicLibraryItem *albumItem, artistItem->childItems()) {
-//                 if (albumItem->data()==album) {
-//                     if (static_cast<const MusicLibraryItemAlbum *>(albumItem)->setCover(img)) {
-//                         QModelIndex idx=index(j, 0, index(i, 0, QModelIndex()));
-//                         emit dataChanged(idx, idx);
-//                     }
-//                     return;
-//                 }
-//                 j++;
-//             }
-//         }
-//         i++;
-//     }
 }
 
 /**
@@ -668,6 +733,7 @@ void MusicLibraryModel::toXML(const MusicLibraryItemRoot *root, const QDateTime
  */
 bool MusicLibraryModel::fromXML()
 {
+    convertCache(cacheFileName());
     MusicLibraryItemRoot *root=new MusicLibraryItemRoot;
     quint32 date=root->fromXML(cacheFileName(), MPDStats::self()->dbUpdate());
     if (!date) {
@@ -763,7 +829,7 @@ QList<Song> MusicLibraryModel::songs(const QModelIndexList &indexes, bool allowP
     return songs;
 }
 
-QList<Song> MusicLibraryModel::songs(const QStringList &filenames) const
+QList<Song> MusicLibraryModel::songs(const QStringList &filenames, bool insertNotFound) const
 {
     QList<Song> songs;
 
@@ -786,6 +852,13 @@ QList<Song> MusicLibraryModel::songs(const QStringList &filenames) const
         }
     }
 
+    if (insertNotFound && files.size()) {
+        foreach (const QString &file, files) {
+            Song s;
+            s.file=file;
+            songs.append(s);
+        }
+    }
     return songs;
 }
 
