@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -24,13 +24,14 @@
  * along with QtMPC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QtGui/QPalette>
-#include <QtGui/QFont>
-#include <QtCore/QModelIndex>
-#include <QtCore/QMimeData>
-#include <QtCore/QTextStream>
-#include <QtCore/QSet>
-#include <QtCore/QUrl>
+#include <QPalette>
+#include <QFont>
+#include <QModelIndex>
+#include <QMimeData>
+#include <QTextStream>
+#include <QSet>
+#include <QUrl>
+#include <QTimer>
 #include "localize.h"
 #include "playqueuemodel.h"
 #include "groupedview.h"
@@ -39,6 +40,7 @@
 #include "mpdstats.h"
 #include "mpdstatus.h"
 #include "streamfetcher.h"
+#include "streamsmodel.h"
 #ifdef TAGLIB_FOUND
 #include "httpserver.h"
 #endif
@@ -46,19 +48,25 @@
 #include "icon.h"
 #include "config.h"
 
-static QStringList reverseList(const QStringList &orig)
-{
-    QStringList rev;
-    foreach (const QString &s, orig) {
-        rev.prepend(s);
-    }
-    return rev;
-}
-
 const QLatin1String PlayQueueModel::constMoveMimeType("cantata/move");
 const QLatin1String PlayQueueModel::constFileNameMimeType("cantata/filename");
 const QLatin1String PlayQueueModel::constUriMimeType("text/uri-list");
 
+#ifdef TAGLIB_FOUND
+static bool checkExtension(const QString &file)
+{
+    static QSet<QString> constExtensions=QSet<QString>()
+            << QLatin1String("mp3") << QLatin1String("ogg") << QLatin1String("flac") << QLatin1String("wma") << QLatin1String("m4a")
+            << QLatin1String("m4b") << QLatin1String("mp4") << QLatin1String("m4p") << QLatin1String("wav") << QLatin1String("wv")
+            << QLatin1String("wvp") << QLatin1String("aiff") << QLatin1String("aif") << QLatin1String("aifc") << QLatin1String("ape")
+            << QLatin1String("spx") << QLatin1String("tta") << QLatin1String("mpc") << QLatin1String("mpp") << QLatin1String("mp+")
+            << QLatin1String("dff") << QLatin1String("dsf");
+
+    int pos=file.lastIndexOf('.');
+    return pos>1 ? constExtensions.contains(file.mid(pos+1).toLower()) : false;
+}
+#endif
+
 void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QStringList &values)
 {
     QByteArray encodedData;
@@ -105,18 +113,22 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     , currentSongId(-1)
     , currentSongRowNum(-1)
     , mpdState(MPDState_Inactive)
-    , grouped(false)
     , dropAdjust(0)
+    , stopAfterCurrent(false)
+    , stopAfterTrackId(-1)
 {
     fetcher=new StreamFetcher(this);
     connect(this, SIGNAL(modelReset()), this, SLOT(stats()));
     connect(fetcher, SIGNAL(result(const QStringList &, int, bool, quint8)), SLOT(addFiles(const QStringList &, int, bool, quint8)));
+    connect(fetcher, SIGNAL(result(const QStringList &, int, bool, quint8)), SIGNAL(streamsFetched()));
     connect(this, SIGNAL(filesAdded(const QStringList, quint32, quint32, bool, quint8)),
-            MPDConnection::self(), SLOT(addid(const QStringList, quint32, quint32, bool, quint8)));
+            MPDConnection::self(), SLOT(add(const QStringList, quint32, quint32, bool, quint8)));
     connect(this, SIGNAL(move(const QList<quint32> &, quint32, quint32)),
             MPDConnection::self(), SLOT(move(const QList<quint32> &, quint32, quint32)));
-    connect(MPDConnection::self(), SIGNAL(prioritySet(const QList<quint32> &, quint8)),
-            this, SLOT(prioritySet(const QList<quint32> &, quint8)));
+    connect(MPDConnection::self(), SIGNAL(prioritySet(const QList<qint32> &, quint8)), SLOT(prioritySet(const QList<qint32> &, quint8)));
+    connect(MPDConnection::self(), SIGNAL(stopAfterCurrentChanged(bool)), SLOT(stopAfterCurrentChanged(bool)));
+    connect(this, SIGNAL(stop(bool)), MPDConnection::self(), SLOT(stopPlaying(bool)));
+    connect(this, SIGNAL(clearStopAfter()), MPDConnection::self(), SLOT(clearStopAfter()));
 }
 
 PlayQueueModel::~PlayQueueModel()
@@ -125,7 +137,7 @@ PlayQueueModel::~PlayQueueModel()
 
 QModelIndex PlayQueueModel::index(int row, int column, const QModelIndex &parent) const
 {
-    return hasIndex(row, column, parent) ? createIndex(row, column, 0) : QModelIndex();
+    return hasIndex(row, column, parent) ? createIndex(row, column, (void *)&songs.at(row)) : QModelIndex();
 }
 
 QModelIndex PlayQueueModel::parent(const QModelIndex &idx) const
@@ -161,14 +173,9 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
     return QVariant();
 }
 
-int PlayQueueModel::rowCount(const QModelIndex &) const
+int PlayQueueModel::rowCount(const QModelIndex &idx) const
 {
-    return songs.size();
-}
-
-int PlayQueueModel::columnCount(const QModelIndex &) const
-{
-    return grouped ? 1 : COL_COUNT;
+    return idx.isValid() ? 0 : songs.size();
 }
 
 QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
@@ -253,24 +260,30 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
                 switch (mpdState) {
                 case MPDState_Inactive:
                 case MPDState_Stopped: return (int)GroupedView::State_Stopped;
-                case MPDState_Playing: return (int)GroupedView::State_Playing;
+                case MPDState_Playing: return (int)(stopAfterCurrent ? GroupedView::State_StopAfter : GroupedView::State_Playing);
                 case MPDState_Paused:  return (int)GroupedView::State_Paused;
                 }
+            }  else if (-1!=s.id && s.id==stopAfterTrackId) {
+                return GroupedView::State_StopAfterTrack;
             }
         }
         return QVariant();
     }
-    case GroupedView::Role_Status:
-        if (songs.at(index.row()).id == currentSongId) {
+    case GroupedView::Role_Status: {
+        qint32 id=songs.at(index.row()).id;
+        if (id==currentSongId) {
             switch (mpdState) {
             case MPDState_Inactive:
             case MPDState_Stopped: return (int)GroupedView::State_Stopped;
-            case MPDState_Playing: return (int)GroupedView::State_Playing;
+            case MPDState_Playing: return (int)(stopAfterCurrent ? GroupedView::State_StopAfter : GroupedView::State_Playing);
             case MPDState_Paused:  return (int)GroupedView::State_Paused;
             }
+        } else if (-1!=id && id==stopAfterTrackId) {
+            return GroupedView::State_StopAfterTrack;
         }
         return (int)GroupedView::State_Default;
         break;
+    }
     case Qt::FontRole: {
         Song s=songs.at(index.row());
 
@@ -358,12 +371,17 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
             return int(Qt::AlignVCenter|Qt::AlignRight);
         }
     case Qt::DecorationRole:
-        if (COL_STATUS==index.column() && songs.at(index.row()).id == currentSongId) {
-            switch (mpdState) {
-            case MPDState_Inactive:
-            case MPDState_Stopped: return Icon("media-playback-stop");
-            case MPDState_Playing: return Icon("media-playback-start");
-            case MPDState_Paused:  return Icon("media-playback-pause");
+        if (COL_STATUS==index.column()) {
+            qint32 id=songs.at(index.row()).id;
+            if (id==currentSongId) {
+                switch (mpdState) {
+                case MPDState_Inactive:
+                case MPDState_Stopped: return Icon("media-playback-stop");
+                case MPDState_Playing: return Icon(stopAfterCurrent ? "media-playback-stop" : "media-playback-start");
+                case MPDState_Paused:  return Icon("media-playback-pause");
+                }
+            } else if (-1!=id && id==stopAfterTrackId) {
+                return Icon("media-playback-stop");
             }
         }
         break;
@@ -486,12 +504,12 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
         return true;
     } else if (data->hasFormat(constFileNameMimeType)) {
         //Act on moves from the music library and dir view
-        addItems(reverseList(decode(*data, constFileNameMimeType)), row, false, 0);
+        addItems(decode(*data, constFileNameMimeType), row, false, 0);
         return true;
     }
     #ifdef TAGLIB_FOUND
     else if(data->hasFormat(constUriMimeType)/* && MPDConnection::self()->getDetails().isLocal()*/) {
-        QStringList orig=reverseList(decode(*data, constUriMimeType));
+        QStringList orig=decode(*data, constUriMimeType);
         QStringList useable;
         bool haveHttp=HttpServer::self()->isAlive();
         bool alwaysUseHttp=haveHttp && Settings::self()->alwaysUseHttp();
@@ -505,10 +523,12 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
                 if (u.startsWith(QLatin1String("file://"))) {
                     u=u.mid(7);
                 }
-                if (alwaysUseHttp || !mpdLocal) {
-                    useable.append(HttpServer::self()->encodeUrl(QUrl::fromPercentEncoding(u.toUtf8())));
-                } else {
-                    useable.append(QLatin1String("file://")+QUrl::fromPercentEncoding(u.toUtf8()));
+                if (checkExtension(u)) {
+                    if (alwaysUseHttp || !mpdLocal) {
+                        useable.append(HttpServer::self()->encodeUrl(QUrl::fromPercentEncoding(u.toUtf8())));
+                    } else {
+                        useable.append(QLatin1String("file://")+QUrl::fromPercentEncoding(u.toUtf8()));
+                    }
                 }
             }
         }
@@ -523,18 +543,19 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
 
 void PlayQueueModel::addItems(const QStringList &items, int row, bool replace, quint8 priority)
 {
-    bool haveHttp=false;
+    bool haveRadioStream=false;
 
     foreach (const QString &f, items) {
         QUrl u(f);
 
-        if (u.scheme()=="cantata-http") {
-            haveHttp=true;
+        if (u.scheme().startsWith(StreamsModel::constPrefix)) {
+            haveRadioStream=true;
             break;
         }
     }
 
-    if (haveHttp) {
+    if (haveRadioStream) {
+        emit fetchingStreams();
         fetcher->get(items, row, replace, priority);
     } else {
         addFiles(items, row, replace, priority);
@@ -544,7 +565,7 @@ void PlayQueueModel::addItems(const QStringList &items, int row, bool replace, q
 void PlayQueueModel::addFiles(const QStringList &filenames, int row, bool replace, quint8 priority)
 {
     //Check for empty playlist
-    if (replace || (songs.size() == 1 && songs.at(0).artist.isEmpty() && songs.at(0).album.isEmpty() && songs.at(0).title.isEmpty())) {
+    if (replace || songs.isEmpty()) {
         emit filesAdded(filenames, 0, 0, replace, priority);
     } else {
         if (row < 0) {
@@ -555,9 +576,9 @@ void PlayQueueModel::addFiles(const QStringList &filenames, int row, bool replac
     }
 }
 
-void PlayQueueModel::prioritySet(const QList<quint32> &ids, quint8 priority)
+void PlayQueueModel::prioritySet(const QList<qint32> &ids, quint8 priority)
 {
-    QSet<quint32> i=ids.toSet();
+    QSet<qint32> i=ids.toSet();
     int row=0;
 
     foreach (const Song &s, songs) {
@@ -597,7 +618,7 @@ qint32 PlayQueueModel::getRowById(qint32 id) const
 
 Song PlayQueueModel::getSongByRow(const qint32 row) const
 {
-    return row>=songs.size() ? Song() : songs.at(row);
+    return row<0 || row>=songs.size() ? Song() : songs.at(row);
 }
 
 void PlayQueueModel::updateCurrentSong(quint32 id)
@@ -612,12 +633,21 @@ void PlayQueueModel::updateCurrentSong(quint32 id)
 
     currentSongRowNum=getRowById(currentSongId);
     emit dataChanged(index(currentSongRowNum, 0), index(currentSongRowNum, columnCount(QModelIndex())-1));
+
+    if (-1!=currentSongId && stopAfterTrackId==currentSongId) {
+        stopAfterTrackId=-1;
+        emit stop(true);
+    }
 }
 
 void PlayQueueModel::clear()
 {
     beginResetModel();
-    songs=QList<Song>();
+    songs.clear();
+    ids.clear();
+    currentSongId=-1;
+    currentSongRowNum=0;
+    stopAfterTrackId=-1;
     endResetModel();
 }
 
@@ -642,11 +672,6 @@ void PlayQueueModel::setState(MPDState st)
     }
 }
 
-void PlayQueueModel::setGrouped(bool g)
-{
-    grouped=g;
-}
-
 // Update playqueue with contents returned from MPD.
 void PlayQueueModel::update(const QList<Song> &songList)
 {
@@ -660,6 +685,9 @@ void PlayQueueModel::update(const QList<Song> &songList)
         songs=songList;
         ids=newIds;
         endResetModel();
+        if (songList.isEmpty()) {
+            stopAfterTrackId=-1;
+        }
     } else {
         quint32 time = 0;
 
@@ -713,10 +741,29 @@ void PlayQueueModel::update(const QList<Song> &songList)
         }
 
         ids=newIds;
+        if (-1!=stopAfterTrackId && !ids.contains(stopAfterTrackId)) {
+            stopAfterTrackId=-1;
+        }
         emit statsUpdated(songs.size(), time);
     }
 }
 
+void PlayQueueModel::setStopAfterTrack(qint32 track)
+{
+    bool clear=track==stopAfterTrackId || (track==currentSongId && stopAfterCurrent);
+
+    stopAfterTrackId=clear ? -1 : track;
+    qint32 row=getRowById(track);
+    if (-1!=row) {
+        emit dataChanged(index(row, 0), index(row, columnCount(QModelIndex())-1));
+    }
+    if (clear) {
+        emit clearStopAfter();
+    } else if (stopAfterTrackId==currentSongId) {
+        emit stop(true);
+    }
+}
+
 void PlayQueueModel::stats()
 {
     quint32 time = 0;
@@ -729,7 +776,15 @@ void PlayQueueModel::stats()
     emit statsUpdated(songs.size(), time);
 }
 
-QSet<qint32>  PlayQueueModel::getSongIdSet()
+void PlayQueueModel::stopAfterCurrentChanged(bool afterCurrent)
+{
+    if (afterCurrent!=stopAfterCurrent) {
+        stopAfterCurrent=afterCurrent;
+        emit dataChanged(index(currentSongRowNum, 0), index(currentSongRowNum, columnCount(QModelIndex())-1));
+    }
+}
+
+QSet<qint32> PlayQueueModel::getSongIdSet()
 {
     QSet<qint32> ids;
 
@@ -739,3 +794,12 @@ QSet<qint32>  PlayQueueModel::getSongIdSet()
 
     return ids;
 }
+
+QStringList PlayQueueModel::filenames()
+{
+    QStringList names;
+    foreach(const Song &song, songs) {
+        names << song.file;
+    }
+    return names;
+}
