@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -27,14 +27,15 @@
 #include "mpdconnection.h"
 #include "mpdparseutils.h"
 #include "localize.h"
+#include "utils.h"
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobal>
 #endif
-#include <QtGui/QApplication>
-#include <QtCore/QDebug>
-#include <QtCore/QThread>
-#include <QtCore/QStringList>
-#include <QtCore/QTimer>
+#include <QApplication>
+#include <QDebug>
+#include <QStringList>
+#include <QTimer>
+#include "thread.h"
 #include "settings.h"
 
 // #define DBUG qWarning() << "MPDConnection" << QThread::currentThreadId()
@@ -77,7 +78,7 @@ static QByteArray readFromSocket(MpdSocket &socket)
                 break;
             }
             if (++attempt>=constMaxReadAttempts) {
-                qWarning() << "ERROR: Timedout waiting for response";
+                DBUG << "ERROR: Timedout waiting for response";
                 socket.close();
                 return QByteArray();
             }
@@ -142,12 +143,19 @@ QString MPDConnectionDetails::description() const
 }
 
 MPDConnection::MPDConnection()
-    : ver(0)
+    : thread(0)
+    , ver(0)
     , sock(this)
     , idleSocket(this)
+    , lastStatusPlayQueueVersion(0)
+    , lastUpdatePlayQueueVersion(0)
     , state(State_Blank)
     , reconnectTimer(0)
     , reconnectStart(0)
+    , stopAfterCurrent(false)
+    , currentSongId(-1)
+    , songPos(0)
+    , unmuteVol(-1)
 {
     qRegisterMetaType<Song>("Song");
     qRegisterMetaType<Output>("Output");
@@ -172,6 +180,23 @@ MPDConnection::~MPDConnection()
     idleSocket.disconnectFromHost();
 }
 
+void MPDConnection::start()
+{
+    if (!thread) {
+        thread=new Thread(metaObject()->className());
+        moveToThread(thread);
+        thread->start();
+    }
+}
+
+void MPDConnection::stop()
+{
+    if (thread) {
+        thread->stop();
+        thread=0;
+    }
+}
+
 MPDConnection::ConnectionReturn MPDConnection::connectToMPD(MpdSocket &socket, bool enableIdle)
 {
     if (QAbstractSocket::ConnectedState!=socket.state()) {
@@ -202,7 +227,6 @@ MPDConnection::ConnectionReturn MPDConnection::connectToMPD(MpdSocket &socket, b
                 long v=((maj&0xFF)<<16)+((min&0xFF)<<8)+(patch&0xFF);
                 if (v!=ver) {
                     ver=v;
-                    emit version(ver);
                 }
             }
 
@@ -342,6 +366,8 @@ void MPDConnection::setDetails(const MPDConnectionDetails &det)
         DBUG << "setDetails" << details.hostname << details.port << (details.password.isEmpty() ? false : true);
         disconnectFromMPD();
         DBUG << "call connectToMPD";
+        unmuteVol=-1;
+        toggleStopAfterCurrent(false);
         switch (connectToMPD()) {
         case Success:
             getStatus();
@@ -415,9 +441,8 @@ MPDConnection::Response MPDConnection::sendCommand(const QByteArray &command, bo
         DBUG << command << "failed";
         if (response.data.isEmpty() && retry && QAbstractSocket::ConnectedState!=sock.state()) {
             // Try one more time...
-            // This scenario, where socket seems to be closed during/after 'write' seems to occor more often
+            // This scenario, where socket seems to be closed during/after 'write' seems to occiu more often
             // when dynamizer is running. However, simply reconnecting seems to resolve the issue.
-            qWarning() << "Socket is not connected, so try reconnecting and re-send command";
             return sendCommand(command, emitErrors, false);
         }
         if (emitErrors) {
@@ -458,16 +483,23 @@ bool MPDConnection::isPlaylist(const QString &file)
 
 void MPDConnection::add(const QStringList &files, bool replace, quint8 priority)
 {
+    add(files, 0, 0, replace, priority);
+}
+
+void MPDConnection::add(const QStringList &files, quint32 pos, quint32 size, bool replace, quint8 priority)
+{
+    toggleStopAfterCurrent(false);
     if (replace) {
         clear();
         getStatus();
     }
 
     QByteArray send = "command_list_begin\n";
+    int curSize = size;
+    int curPos = pos;
     bool addedFile=false;
     bool havePlaylist=false;
     bool usePrio=priority>0 && canUsePriority();
-    quint32 pos=playQueueIds.count();
     for (int i = 0; i < files.size(); i++) {
         if (isPlaylist(files.at(i))) {
             send+="load ";
@@ -477,9 +509,16 @@ void MPDConnection::add(const QStringList &files, bool replace, quint8 priority)
             send += "add ";
         }
         send += encodeName(files.at(i))+"\n";
-        if (usePrio && !havePlaylist) {
-            send += "prio "+QByteArray::number(priority)+" "+QByteArray::number(pos+i)+" "+QByteArray::number(pos+i)+"\n";
+        if (!havePlaylist) {
+            if (0!=size) {
+                send += "move "+QByteArray::number(curSize)+" "+QByteArray::number(curPos)+"\n";
+            }
+            if (usePrio && !havePlaylist) {
+                send += "prio "+QByteArray::number(priority)+" "+QByteArray::number(curPos)+" "+QByteArray::number(curPos)+"\n";
+            }
         }
+        curSize++;
+        curPos++;
     }
 
     send += "command_list_end";
@@ -492,55 +531,9 @@ void MPDConnection::add(const QStringList &files, bool replace, quint8 priority)
     }
 }
 
-/**
- * Add all the files in the QStringList to the playlist at
- * postition post.
- *
- * NOTE: addid is not fully supported in < 0.14 So add everything
- * and then move everything to the right spot.
- *
- * @param files A QStringList with all the files to add
- * @param pos Position to add the files
- * @param size The size of the current playlist
- */
-void MPDConnection::addid(const QStringList &files, quint32 pos, quint32 size, bool replace, quint8 priority)
-{
-    if (replace) {
-        clear();
-        getStatus();
-    }
-
-    QByteArray send = "command_list_begin\n";
-    int curSize = size;
-    bool havePlaylist=false;
-    bool usePrio=priority>0 && canUsePriority();
-    for (int i = 0; i < files.size(); i++) {
-        if (isPlaylist(files.at(i))) {
-            send+="load ";
-            havePlaylist=true;
-        } else {
-            send += "add ";
-        }
-        send += encodeName(files.at(i));
-        send += "\n";
-        if (!havePlaylist) {
-            send += "move "+QByteArray::number(curSize)+" "+QByteArray::number(pos)+"\n";
-            if (usePrio) {
-                send += "prio "+QByteArray::number(priority)+" "+QByteArray::number(pos)+" "+QByteArray::number(pos)+"\n";
-            }
-        }
-        curSize++;
-    }
-
-    send += "command_list_end";
-
-    if (sendCommand(send).ok) {
-        emit added(files);
-    }
-}
-
 void MPDConnection::clear()
 {
+    toggleStopAfterCurrent(false);
     if (sendCommand("clear").ok) {
         lastUpdatePlayQueueVersion=0;
         playQueueIds.clear();
@@ -549,8 +542,8 @@ void MPDConnection::clear()
 
 void MPDConnection::removeSongs(const QList<qint32> &items)
 {
+    toggleStopAfterCurrent(false);
     QByteArray send = "command_list_begin\n";
-
     foreach (qint32 i, items) {
         send += "deleteid ";
         send += QByteArray::number(i);
@@ -563,6 +556,7 @@ void MPDConnection::removeSongs(const QList<qint32> &items)
 
 void MPDConnection::move(quint32 from, quint32 to)
 {
+    toggleStopAfterCurrent(false);
     sendCommand("move "+QByteArray::number(from)+' '+QByteArray::number(to));
 }
 
@@ -606,11 +600,13 @@ void MPDConnection::move(const QList<quint32> &items, quint32 pos, quint32 size)
 
 void MPDConnection::shuffle()
 {
+    toggleStopAfterCurrent(false);
     sendCommand("shuffle");
 }
 
 void MPDConnection::shuffle(quint32 from, quint32 to)
 {
+    toggleStopAfterCurrent(false);
     sendCommand("shuffle "+QByteArray::number(from)+':'+QByteArray::number(to+1));
 }
 
@@ -618,7 +614,7 @@ void MPDConnection::currentSong()
 {
     Response response=sendCommand("currentsong");
     if (response.ok) {
-        emit currentSongUpdated(MPDParseUtils::parseSong(response.data));
+        emit currentSongUpdated(MPDParseUtils::parseSong(response.data, true));
     }
 }
 
@@ -685,8 +681,7 @@ void MPDConnection::playListChanges()
                         playListInfo();
                         return;
                     }
-                    Song s=MPDParseUtils::parseSong(response.data);
-                    s.setKey();
+                    Song s=MPDParseUtils::parseSong(response.data, true);
                     s.id=idp.id;
 //                     s.pos=idp.pos;
                     songs.append(s);
@@ -765,6 +760,7 @@ void MPDConnection::getReplayGain()
 
 void MPDConnection::goToNext()
 {
+    toggleStopAfterCurrent(false);
     sendCommand("next");
 }
 
@@ -775,21 +771,25 @@ static inline QByteArray value(bool b)
 
 void MPDConnection::setPause(bool toggle)
 {
+    toggleStopAfterCurrent(false);
     sendCommand("pause "+value(toggle));
 }
 
 void MPDConnection::startPlayingSong(quint32 song)
 {
+    toggleStopAfterCurrent(false);
     sendCommand("play "+QByteArray::number(song));
 }
 
-void MPDConnection::startPlayingSongId(quint32 songId)
+void MPDConnection::startPlayingSongId(qint32 songId)
 {
+    toggleStopAfterCurrent(false);
     sendCommand("playid "+QByteArray::number(songId));
 }
 
 void MPDConnection::goToPrevious()
 {
+    toggleStopAfterCurrent(false);
     sendCommand("previous");
 }
 
@@ -815,24 +815,64 @@ void MPDConnection::setSingle(bool toggle)
 
 void MPDConnection::setSeek(quint32 song, quint32 time)
 {
+    toggleStopAfterCurrent(false);
     sendCommand("seek "+QByteArray::number(song)+' '+QByteArray::number(time));
 }
 
-void MPDConnection::setSeekId(quint32 songId, quint32 time)
+void MPDConnection::setSeekId(qint32 songId, quint32 time)
 {
-    sendCommand("seekid "+QByteArray::number(songId)+' '+QByteArray::number(time));
+    if (-1==songId) {
+        songId=currentSongId;
+    }
+    if (-1==songId) {
+        return;
+    }
+    if (songId!=currentSongId || 0==time) {
+        toggleStopAfterCurrent(false);
+    }
+    if (sendCommand("seekid "+QByteArray::number(songId)+' '+QByteArray::number(time)).ok) {
+        if (stopAfterCurrent && songId==currentSongId && songPos>time) {
+            songPos=time;
+        }
+    }
 }
 
 void MPDConnection::setVolume(int vol)
 {
     if (vol>=0) {
-        sendCommand("setvol "+QByteArray::number(vol));
+        unmuteVol=-1;
+        sendCommand("setvol "+QByteArray::number(vol), false);
     }
 }
 
-void MPDConnection::stopPlaying()
+void MPDConnection::toggleMute()
 {
-    sendCommand("stop");
+    if (unmuteVol>0) {
+        sendCommand("setvol "+QByteArray::number(unmuteVol), false);
+        unmuteVol=-1;
+    } else {
+        Response status=sendCommand("status");
+        if (status.ok) {
+            MPDStatusValues sv=MPDParseUtils::parseStatus(status.data);
+            if (sv.volume>0) {
+                unmuteVol=sv.volume;
+                sendCommand("setvol "+QByteArray::number(0), false);
+            }
+        }
+    }
+}
+
+void MPDConnection::stopPlaying(bool afterCurrent)
+{
+    toggleStopAfterCurrent(afterCurrent);
+    if (!stopAfterCurrent) {
+        sendCommand("stop");
+    }
+}
+
+void MPDConnection::clearStopAfter()
+{
+    toggleStopAfterCurrent(false);
 }
 
 void MPDConnection::getStats(bool andUpdate)
@@ -851,6 +891,13 @@ void MPDConnection::getStatus()
     if (response.ok) {
         MPDStatusValues sv=MPDParseUtils::parseStatus(response.data);
         lastStatusPlayQueueVersion=sv.playlist;
+        if (stopAfterCurrent && (currentSongId!=sv.songId || (songPos>0 && sv.timeElapsed<(qint32)songPos))) {
+            if (sendCommand("stop").ok) {
+                sv.state=MPDState_Stopped;
+            }
+            toggleStopAfterCurrent(false);
+        }
+        currentSongId=sv.songId;
         emit statusUpdated(sv);
     }
 }
@@ -859,7 +906,7 @@ void MPDConnection::getUrlHandlers()
 {
     Response response=sendCommand("urlhandlers");
     if (response.ok) {
-        emit urlHandlers(MPDParseUtils::parseUrlHandlers(response.data));
+        handlers=MPDParseUtils::parseUrlHandlers(response.data).toSet();
     }
 }
 
@@ -870,12 +917,9 @@ void MPDConnection::getUrlHandlers()
 void MPDConnection::idleDataReady()
 {
     DBUG << "idleDataReady";
-    QByteArray data;
-
     if (idleSocket.bytesAvailable() == 0) {
         return;
     }
-
     parseIdleReturn(readFromSocket(idleSocket));
 }
 
@@ -949,8 +993,9 @@ void MPDConnection::parseIdleReturn(const QByteArray &data)
             if (!playListUpdated) {
                 playListChanges();
             }
-        } else if (!statusUpdated && (line == "changed: player" || line == "changed: mixer" || line == "changed: options" )) {
+        } else if (!statusUpdated && (line == "changed: player" || line == "changed: mixer" || line == "changed: options")) {
             getStatus();
+            getReplayGain();
             statusUpdated=true;
         } else if (line == "changed: output") {
             outputs();
@@ -1131,7 +1176,7 @@ void MPDConnection::removeFromPlaylist(const QString &name, const QList<quint32>
 //     playlistInfo(name);
 }
 
-void MPDConnection::setPriority(const QList<quint32> &ids, quint8 priority)
+void MPDConnection::setPriority(const QList<qint32> &ids, quint8 priority)
 {
     if (canUsePriority()) {
         QByteArray send = "command_list_begin\n";
@@ -1157,6 +1202,9 @@ void MPDConnection::moveInPlaylist(const QString &name, const QList<quint32> &it
 
 bool MPDConnection::doMoveInPlaylist(const QString &name, const QList<quint32> &items, quint32 pos, quint32 size)
 {
+    if (name.isEmpty()) {
+        toggleStopAfterCurrent(false);
+    }
     QByteArray cmd = name.isEmpty() ? "move " : ("playlistmove "+encodeName(name)+" ");
     QByteArray send = "command_list_begin\n";
     QList<quint32> moveItems=items;
@@ -1189,6 +1237,22 @@ bool MPDConnection::doMoveInPlaylist(const QString &name, const QList<quint32> &
     return sendCommand(send).ok;
 }
 
+void MPDConnection::toggleStopAfterCurrent(bool afterCurrent)
+{
+    if (afterCurrent!=stopAfterCurrent) {
+        stopAfterCurrent=afterCurrent;
+        songPos=0;
+        if (stopAfterCurrent && 1==playQueueIds.count()) {
+            Response response=sendCommand("status");
+            if (response.ok) {
+                MPDStatusValues sv=MPDParseUtils::parseStatus(response.data);
+                songPos=sv.timeElapsed;
+            }
+        }
+        emit stopAfterCurrentChanged(stopAfterCurrent);
+    }
+}
+
 MpdSocket::MpdSocket(QObject *parent)
     : QObject(parent)
     , tcp(0)
