@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -33,10 +33,8 @@ class TagReader : public Job
     Q_OBJECT
 
 public:
-    TagReader() {
-    }
-    virtual ~TagReader() {
-    }
+    TagReader() { }
+    virtual ~TagReader() { }
 
     void setDetails(const QList<Song> &s, const QString &dir);
 
