@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  */
 
 /**************************************************************************
@@ -37,25 +37,21 @@
 #include "localize.h"
 #include "icon.h"
 #include "gtkstyle.h"
-// #include "stylehelper.h"
-
-// #include <QtGui/QColorDialog>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QMenu>
-#include <QtGui/QMouseEvent>
-#include <QtGui/QPainter>
-#include <QtGui/QSplitter>
-#include <QtGui/QStackedLayout>
-#include <QtGui/QStyleOptionTabV3>
-#include <QtGui/QStyleOptionViewItemV4>
-#include <QtGui/QToolButton>
-#include <QtGui/QToolTip>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QWindowsStyle>
-#include <QtGui/QApplication>
-#include <QtCore/QAnimationGroup>
-#include <QtCore/QPropertyAnimation>
-#include <QtCore/QSignalMapper>
+#include "action.h"
+#include <QHBoxLayout>
+#include <QMenu>
+#include <QMouseEvent>
+#include <QPainter>
+#include <QSplitter>
+#include <QStackedLayout>
+#include <QStyleOption>
+#include <QToolButton>
+#include <QToolTip>
+#include <QVBoxLayout>
+#include <QApplication>
+#include <QAnimationGroup>
+#include <QPropertyAnimation>
+#include <QSignalMapper>
 
 static inline Qt::TextElideMode elideMode()
 {
@@ -125,9 +121,9 @@ static void drawRoundedRect(QPainter *p, const QRect &r, double radius, const QC
 }
 #endif
 
-static void drawIcon(const QIcon &icon, const QRect &r, QPainter *p, const QSize &iconSize)
+static void drawIcon(const QIcon &icon, const QRect &r, QPainter *p, const QSize &iconSize, bool selected)
 {
-    QPixmap px = icon.pixmap(iconSize);
+    QPixmap px = icon.pixmap(iconSize, selected ? QIcon::Selected : QIcon::Normal);
     p->drawPixmap(r.x()+(r.width()-px.width())/2.0, r.y()+(r.height()-px.height())/2.0, px.width(), px.height(), px);
 }
 
@@ -239,9 +235,11 @@ void FancyTabProxyStyle::drawControl(
   int textFlags = Qt::AlignTop | Qt::AlignVCenter;
 //   p->drawText(text_rect, textFlags, text);
 //   p->setPen(selected ? QColor(60, 60, 60) : Utils::StyleHelper::panelTextColor());
-    p->setPen(selected ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
+    p->setPen(selected && option->state&State_Active
+              ? QApplication::palette().highlightedText().color() : QApplication::palette().foreground().color());
 
-  drawIcon(v_opt->icon, icon_rect, p, v_opt->iconSize);
+  drawIcon(v_opt->icon, icon_rect, p, v_opt->iconSize,
+           selected && option->state&State_Active);
 
   QString txt=text;
   txt.replace("&", "");
@@ -320,7 +318,6 @@ FancyTabBar::FancyTabBar(QWidget *parent, bool hasBorder, bool text, int iSize,
     , m_pos(pos)
     , m_iconSize(iSize)
 {
-//    setStyle(new QWindowsStyle);
     setAttribute(Qt::WA_Hover, true);
     setFocusPolicy(Qt::NoFocus);
     setMouseTracking(true); // Needed for hover events
@@ -546,12 +543,15 @@ void FancyTabBar::paintTab(QPainter *painter, int tabIndex, bool gtkStyle) const
         const int textHeight = painter->fontMetrics().height();
         tabIconRect.adjust(0, 4, 0, -textHeight);
 //         Utils::StyleHelper::drawIconWithShadow(tabIcon(tabIndex), tabIconRect, painter, QIcon::Normal);
-        drawIcon(tabIcon(tabIndex), tabIconRect, painter, QSize(m_iconSize, m_iconSize));
+        drawIcon(tabIcon(tabIndex), tabIconRect, painter, QSize(m_iconSize, m_iconSize),
+                 selected && (palette().currentColorGroup()==QPalette::Active ||
+                             (palette().highlightedText().color()==palette().brush(QPalette::Active, QPalette::HighlightedText).color())));
     } else {
-        drawIcon(tabIcon(tabIndex), rect, painter, QSize(m_iconSize, m_iconSize));
+        drawIcon(tabIcon(tabIndex), rect, painter, QSize(m_iconSize, m_iconSize),
+                 selected && (palette().currentColorGroup()==QPalette::Active ||
+                             (palette().highlightedText().color()==palette().brush(QPalette::Active, QPalette::HighlightedText).color())));
     }
 
-
 //     painter->translate(0, -1);
 //     painter->drawText(tabTextRect, textFlags, tabText);
     painter->restore();
@@ -696,22 +696,22 @@ static void drawFadedLine(QPainter *p, const QRect &r, const QColor &col)
 
 void FancyTabWidget::paintEvent(QPaintEvent*e) {
     QWidget::paintEvent(e);
-    bool onTop=Mode_TopBar==mode_ || Mode_IconOnlyTopBar==mode_;
-    bool onBottom=Mode_BottomBar==mode_ || Mode_IconOnlyBottomBar==mode_;
-    if (!drawBorder_ && !onBottom && !onTop) {
+//    bool onTop=Mode_TopBar==mode_ || Mode_IconOnlyTopBar==mode_;
+//    bool onBottom=Mode_BottomBar==mode_ || Mode_IconOnlyBottomBar==mode_;
+    if (!drawBorder_/* && !onBottom && !onTop*/) {
         return;
     }
 
     QPainter painter(this);
 
-    if (onBottom) {
+    /*if (onBottom) {
         QRect r(rect().x(), side_widget_->rect().y()+(side_widget_->height()-tab_bar_->height()),
                 width(), 1);
         drawFadedLine(&painter, r, palette().foreground().color());
     } else if (onTop) {
         QRect r(rect().x(), tab_bar_->height(), width(), 1);
         drawFadedLine(&painter, r, palette().foreground().color());
-    } else {
+    } else*/ {
         QRect rect = side_widget_->rect().adjusted(0, 0, 1, 0);
         rect = style()->visualRect(layoutDirection(), geometry(), rect);
         drawFadedLine(&painter, QRect(rect.x(), rect.y(), 1, rect.height()), palette().foreground().color());
@@ -937,6 +937,7 @@ void FancyTabWidget::contextMenuEvent(QContextMenuEvent* e) {
             action->setCheckable(true);
             action->setChecked(item.enabled_);
             action->setData(idx);
+            Action::initIcon(action);
             connect(action, SIGNAL(triggered()), this, SLOT(ToggleTab()));
         }
         idx++;
@@ -960,9 +961,11 @@ void FancyTabWidget::contextMenuEvent(QContextMenuEvent* e) {
     iconOnlyAct->setCheckable(true);
     iconOnlyAct->setChecked(Mode_IconOnlyLargeSidebar==mode_ || Mode_IconOnlySmallSidebar==mode_ ||
                             Mode_IconOnlySideTabs==mode_ || Mode_IconOnlyTopTabs==mode_ ||
-                            Mode_IconOnlyBotTabs==mode_ || Mode_IconOnlyTopBar==mode_);
+                            Mode_IconOnlyBotTabs==mode_ || Mode_IconOnlyTopBar==mode_ ||
+                            Mode_IconOnlyBottomBar==mode_);
     iconOnlyAct->setData(0);
     connect(iconOnlyAct, SIGNAL(triggered()), this, SLOT(SetMode()));
+    modeMenu->addSeparator();
     modeMenu->addAction(iconOnlyAct);
     modeAct->setMenu(modeMenu);
     modeAct->setData(-1);
@@ -1021,7 +1024,8 @@ void FancyTabWidget::SetMode()
         } else {
             bool iconOnly=Mode_IconOnlyLargeSidebar==mode_ || Mode_IconOnlySmallSidebar==mode_ ||
                           Mode_IconOnlySideTabs==mode_ || Mode_IconOnlyTopTabs==mode_ ||
-                          Mode_IconOnlyBotTabs==mode_ || Mode_IconOnlyBottomBar==mode_;
+                          Mode_IconOnlyBotTabs==mode_ || Mode_IconOnlyBottomBar==mode_ ||
+                          Mode_IconOnlyTopBar==mode_;
             switch (data) {
             case Mode_LargeSidebar: SetMode(iconOnly ? Mode_IconOnlyLargeSidebar : Mode_LargeSidebar); break;
             case Mode_SmallSidebar: SetMode(iconOnly ? Mode_IconOnlySmallSidebar : Mode_SmallSidebar); break;
