@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,13 +31,14 @@
 #include "localize.h"
 #include "icons.h"
 #include "gtkstyle.h"
-#include <QtGui/QStyledItemDelegate>
-#include <QtGui/QApplication>
-#include <QtGui/QFontMetrics>
-#include <QtGui/QPainter>
-#include <QtGui/QLinearGradient>
-#include <QtGui/QAction>
-#include <QtGui/QDropEvent>
+#include <QStyledItemDelegate>
+#include <QApplication>
+#include <QFontMetrics>
+#include <QPainter>
+#include <QLinearGradient>
+#include <QAction>
+#include <QDropEvent>
+#include <QPixmap>
 
 static int constCoverSize=32;
 static int constIconSize=16;
@@ -49,18 +50,7 @@ void GroupedView::setup()
 
     if (height>17) {
         constCoverSize=((int)((height*2)/4))*4;
-        constIconSize=((int)(height/4))*4;
-        if (constIconSize<20) {
-            constIconSize=16;
-        } else if (constIconSize<28) {
-            constIconSize=22;
-        } else if (constIconSize<40) {
-            constIconSize=32;
-        } else if (constIconSize<56) {
-            constIconSize=48;
-        } else {
-            constIconSize=64;
-        }
+        constIconSize=Icon::stdSize(((int)(height/4))*4);
         constBorder=constCoverSize>48 ? 2 : 1;
     } else {
         constCoverSize=32;
@@ -88,11 +78,32 @@ static bool isAlbumHeader(const QModelIndex &index)
     return !index.data(GroupedView::Role_IsCollection).toBool() && AlbumHeader==getType(index);
 }
 
+static QString streamText(const Song &song, const QString &trackTitle, bool useName=true)
+{
+    if (song.album.isEmpty() && song.albumArtist().isEmpty()) {
+        return song.title.isEmpty() && song.name.isEmpty()
+                ? song.file
+                : !useName || song.name.isEmpty()
+                  ? song.title
+                  : song.title.isEmpty()
+                    ? song.name
+                    : (song.title + " - " + song.name);
+    } else if (!song.title.isEmpty() && !song.artist.isEmpty()) {
+        return song.artist + " - " + (!useName || song.name.isEmpty()
+                                        ? song.title
+                                        : song.title.isEmpty()
+                                            ? song.name
+                                            : (song.title + " - " + song.name));
+    } else {
+        return trackTitle;
+    }
+}
+
 class GroupedViewDelegate : public ActionItemDelegate
 {
 public:
-    GroupedViewDelegate(GroupedView *p, QAction *a1, QAction *a2, QAction *t, int actionLevel)
-        : ActionItemDelegate(p, a1, a2, t, actionLevel)
+    GroupedViewDelegate(GroupedView *p)
+        : ActionItemDelegate(p)
         , view(p)
     {
     }
@@ -178,7 +189,7 @@ public:
         int state=index.data(GroupedView::Role_Status).toInt();
         quint32 collection=index.data(GroupedView::Role_CollectionId).toUInt();
         bool selected=option.state&QStyle::State_Selected;
-        bool mouseOver=option.state&QStyle::State_MouseOver;
+        bool mouseOver=underMouse && option.state&QStyle::State_MouseOver;
         bool gtk=mouseOver && GtkStyle::isActive();
 
         if (!isCollection && AlbumHeader==type) {
@@ -215,6 +226,7 @@ public:
         QString track;
         QString duration=song.time>0 ? Song::formattedTime(song.time) : QString();
         bool stream=!isCollection && (song.isStream() && !song.isCantataStream());
+        bool audiocd=stream && song.isCdda();
         bool isEmpty=song.title.isEmpty() & song.artist.isEmpty() && !song.file.isEmpty();
         QString trackTitle=isEmpty
                         ? song.file
@@ -232,13 +244,14 @@ public:
             title=index.data(Qt::DisplayRole).toString();
         } else if (AlbumHeader==type) {
             if (stream) {
-                title=i18n("Streams");
-                if (song.album.isEmpty() && song.albumArtist().isEmpty()) {
-                    track=song.title.isEmpty() && song.name.isEmpty() ? song.file : (song.name.isEmpty() ? song.title : QString("%1 (%2)").arg(song.title).arg(song.name));
-                } else if (!song.title.isEmpty() && !song.artist.isEmpty()) {
-                    track=song.artist + " - " + (song.name.isEmpty() ? song.title : QString("%1 (%2)").arg(song.title).arg(song.name));
+                QModelIndex next=index.sibling(index.row()+1, 0);
+                quint16 nextKey=next.isValid() ? next.data(GroupedView::Role_Key).toUInt() : Song::constNullKey;
+                if (nextKey!=song.key && !song.name.isEmpty()) {
+                    title=song.name;
+                    track=streamText(song, trackTitle, false);
                 } else {
-                    track=trackTitle;
+                    title=audiocd ? i18n("Audio CD") : i18n("Streams");
+                    track=streamText(song, trackTitle);
                 }
             } else if (isEmpty) {
                 title=i18n("Unknown");
@@ -255,13 +268,7 @@ public:
             }
         } else {
             if (stream) {
-                if (song.album.isEmpty() && song.albumArtist().isEmpty()) {
-                    track=song.title.isEmpty() && song.name.isEmpty() ? song.file : (song.name.isEmpty() ? song.title : QString("%1 (%2)").arg(song.title).arg(song.name));
-                } else if (!song.title.isEmpty() && !song.artist.isEmpty()) {
-                    track=song.artist + " - " + (song.name.isEmpty() ? song.title : QString("%1 (%2)").arg(song.title).arg(song.name));
-                } else {
-                    track=trackTitle;
-                }
+                track=streamText(song, trackTitle);
             } else {
                 track=formatNumber(song.track)+QChar(' ')+trackTitle;
             }
@@ -277,7 +284,7 @@ public:
         QRect r(option.rect.adjusted(constBorder+4, constBorder, -(constBorder+4), -constBorder));
         bool rtl=Qt::RightToLeft==QApplication::layoutDirection();
 
-        if (state) {
+        if (state && GroupedView::State_StopAfterTrack!=state) {
             QRectF border(option.rect.x()+1.5, option.rect.y()+1.5, option.rect.width()-3, option.rect.height()-3);
             if (!title.isEmpty()) {
                 border.adjust(0, textHeight+constBorder, 0, 0);
@@ -305,8 +312,8 @@ public:
             if (isCollection) {
                 pix=index.data(Qt::DecorationRole).value<QIcon>().pixmap(constCoverSize, constCoverSize);
             } else {
-                QPixmap *cover=Covers::self()->get(song, constCoverSize);
-                pix=cover ? *cover : (stream ? Icons::streamIcon : Icons::albumIcon).pixmap(constCoverSize, constCoverSize);
+                QPixmap *cover=stream ? 0 : Covers::self()->get(song, constCoverSize);
+                pix=cover ? *cover : (stream && !audiocd ? Icons::streamIcon : Icons::albumIcon).pixmap(constCoverSize, constCoverSize);
             }
 
             if (rtl) {
@@ -354,16 +361,39 @@ public:
             int size=constIconSize-7;
             int hSize=size/2;
             QRect ir(r.x()-(size+6), r.y()+(((r.height()-size)/2.0)+0.5), size, size);
+            QColor inside(option.palette.color(QPalette::Text));
+            QColor border=inside.red()>100 && inside.blue()>100 && inside.green()>100 ? Qt::black : Qt::white;
             switch (state) {
             case GroupedView::State_Stopped:
-                painter->fillRect(ir, Qt::white);
-                painter->fillRect(ir.adjusted(1, 1, -1, -1), Qt::black);
+                painter->fillRect(ir, border);
+                painter->fillRect(ir.adjusted(1, 1, -1, -1), inside);
+                break;
+            case GroupedView::State_StopAfter: {
+                ir.adjust(2, 0, -2, 0);
+                QPoint p1[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+(ir.height())), QPoint(ir.x()-2, ir.y()+(ir.height())) };
+                QPoint p2[5]={ QPoint(ir.x()-1, ir.y()), QPoint(ir.x(), ir.y()), QPoint(ir.x()+(size-hSize)-1, ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()-1), QPoint(ir.x()-1, ir.y()+ir.height()-1) };
+                QPoint p3[3]={ QPoint(ir.x(), ir.y()+1), QPoint(ir.x()+(size-hSize)-2, ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()-2) };
+                painter->save();
+                painter->setPen(border);
+                painter->drawPolygon(p1, 5);
+                painter->drawPolygon(p3, 3);
+                painter->setPen(inside);
+                painter->drawPolygon(p2, 5);
+                painter->restore();
+                break;
+            }
+            case GroupedView::State_StopAfterTrack:
+                painter->setPen(border);
+                painter->drawRect(ir.adjusted(0, 0, -1, -1));
+                painter->drawRect(ir.adjusted(2, 2, -3, -3));
+                painter->setPen(inside);
+                painter->drawRect(ir.adjusted(1, 1, -2, -2));
                 break;
             case GroupedView::State_Paused: {
                 int blockSize=hSize-1;
-                painter->fillRect(ir, Qt::white);
-                painter->fillRect(ir.x()+1, ir.y()+1, blockSize, size-2, Qt::black);
-                painter->fillRect(ir.x()+size-blockSize-1, ir.y()+1, blockSize, size-2, Qt::black);
+                painter->fillRect(ir, border);
+                painter->fillRect(ir.x()+1, ir.y()+1, blockSize, size-2, inside);
+                painter->fillRect(ir.x()+size-blockSize-1, ir.y()+1, blockSize, size-2, inside);
                 break;
             }
             case GroupedView::State_Playing: {
@@ -371,10 +401,10 @@ public:
                 QPoint p1[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+(ir.height()-1)), QPoint(ir.x()-2, ir.y()+(ir.height()-1)) };
                 QPoint p2[5]={ QPoint(ir.x()-2, ir.y()-1), QPoint(ir.x(), ir.y()-1), QPoint(ir.x()+(size-hSize), ir.y()+hSize), QPoint(ir.x(), ir.y()+ir.height()), QPoint(ir.x()-2, ir.y()+ir.height()) };
                 painter->save();
-                painter->setBrush(Qt::white);
-                painter->setPen(Qt::white);
+                painter->setBrush(border);
+                painter->setPen(border);
                 painter->drawPolygon(p1, 5);
-                painter->setBrush(Qt::black);
+                painter->setBrush(inside);
                 painter->drawPolygon(p2, 5);
                 painter->restore();
                 break;
@@ -391,8 +421,8 @@ public:
             painter->drawText(duratioRect, duration, QTextOption(Qt::AlignVCenter|Qt::AlignRight));
         }
 
-        if ((option.state & QStyle::State_MouseOver) && (act1 || act2 || toggle) && hasActions(index, actLevel)) {
-            drawIcons(painter, option.rect, true, rtl, false, index);
+        if (mouseOver) {
+            drawIcons(painter, option.rect, true, rtl, AlbumHeader==type || isCollection ? AP_HBottom : AP_HMiddle, index);
         }
         painter->restore();
     }
@@ -401,11 +431,12 @@ private:
     GroupedView *view;
 };
 
-GroupedView::GroupedView(QWidget *parent)
-    : TreeView(parent)
+GroupedView::GroupedView(QWidget *parent, bool isPlayQueue)
+    : TreeView(parent, isPlayQueue)
     , startClosed(true)
     , autoExpand(true)
     , filterActive(false)
+    , isMultiLevel(false)
     , currentAlbum(Song::constNullKey)
 {
     setContextMenuPolicy(Qt::CustomContextMenu);
@@ -419,15 +450,32 @@ GroupedView::GroupedView(QWidget *parent)
     setSelectionBehavior(SelectRows);
     connect(this, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
     setStyleSheet("QTreeView::branch { border: 0px; }");
+    GroupedViewDelegate *delegate=new GroupedViewDelegate(this);
+    setItemDelegate(delegate);
+    if (isPlayQueue) {
+        // 'underMouse' is used to work-around mouse over issues with some styles.
+        // PlayQueue does not have mouse over - so we never need to check this.
+        delegate->setUnderMouse(true);
+    }
 }
 
 GroupedView::~GroupedView()
 {
 }
 
-void GroupedView::init(QAction *a1, QAction *a2, QAction *t, int actionLevel)
+void GroupedView::setModel(QAbstractItemModel *model)
 {
-    setItemDelegate(new GroupedViewDelegate(this, a1, a2, t, actionLevel));
+    QTreeView::setModel(model);
+    if (model) {
+        connect(Covers::self(), SIGNAL(coverRetrieved(const Song &)), this, SLOT(coverRetrieved(const Song &)));
+        int columnCount=model->columnCount();
+        QHeaderView *hdr=header();
+        for (int i=1; i<columnCount; ++i) {
+            hdr->setSectionHidden(i, true);
+        }
+    } else {
+        disconnect(Covers::self(), SIGNAL(coverRetrieved(const Song &)), this, SLOT(coverRetrieved(const Song &)));
+    }
 }
 
 void GroupedView::setFilterActive(bool f)
@@ -440,11 +488,13 @@ void GroupedView::setFilterActive(bool f)
         quint32 count=model()->rowCount();
         for (quint32 i=0; i<count; ++i) {
             setRowHidden(i, QModelIndex(), false);
-            QModelIndex idx=model()->index(i, 0);
-            if (model()->hasChildren(idx)) {
-                quint32 childCount=model()->rowCount(idx);
-                for (quint32 c=0; c<childCount; ++c) {
-                    setRowHidden(c, idx, false);
+            if (isMultiLevel) {
+                QModelIndex idx=model()->index(i, 0);
+                if (model()->hasChildren(idx)) {
+                    quint32 childCount=model()->rowCount(idx);
+                    for (quint32 c=0; c<childCount; ++c) {
+                        setRowHidden(c, idx, false);
+                    }
                 }
             }
         }
@@ -460,9 +510,9 @@ void GroupedView::setStartClosed(bool sc)
     startClosed=sc;
 }
 
-void GroupedView::updateRows(qint32 row, bool scroll, const QModelIndex &parent)
+void GroupedView::updateRows(qint32 row, quint16 curAlbum, bool scroll, const QModelIndex &parent)
 {
-    currentAlbum=Song::constNullKey;
+    currentAlbum=curAlbum;
 
     if (row<0) {
         row=0;
@@ -479,7 +529,6 @@ void GroupedView::updateRows(qint32 row, bool scroll, const QModelIndex &parent)
         return;
     }
 
-    currentAlbum=model()->index(row, 0, parent).data(GroupedView::Role_Key).toUInt();
     updateRows(parent);
     if (MPDState_Playing==MPDStatus::self()->state() && scroll) {
         scrollTo(model()->index(row, 0, parent), QAbstractItemView::PositionAtCenter);
@@ -564,7 +613,7 @@ void GroupedView::toggle(const QModelIndex &idx)
 // are selected.
 QModelIndexList GroupedView::selectedIndexes() const
 {
-    QModelIndexList indexes = TreeView::selectedIndexes();
+    QModelIndexList indexes = selectionModel()->selectedRows();
     QSet<QModelIndex> allIndexes;
 
     foreach (const QModelIndex &idx, indexes) {
@@ -641,7 +690,7 @@ void GroupedView::dropEvent(QDropEvent *event)
 
 void GroupedView::coverRetrieved(const Song &s)
 {
-    if (filterActive) {
+    if (filterActive || !isVisible()) {
         return;
     }
 
@@ -653,7 +702,7 @@ void GroupedView::coverRetrieved(const Song &s)
         if (!index.isValid()) {
             continue;
         }
-        if (model()->hasChildren(index)) {
+        if (isMultiLevel && model()->hasChildren(index)) {
             quint32 childCount=model()->rowCount(index);
             for (quint32 c=0; c<childCount; ++c) {
                 QModelIndex child=model()->index(i, 0, index);
@@ -705,7 +754,7 @@ void GroupedView::itemClicked(const QModelIndex &idx)
             QModelIndexList list;
             unsigned int key=idx.data(GroupedView::Role_Key).toUInt();
             QModelIndex i=idx.sibling(idx.row()+1, 0);
-            QModelIndexList sel=selectedIndexes();
+//            QModelIndexList sel=selectedIndexes();
             QItemSelectionModel *selModel=selectionModel();
             QModelIndexList unsel;
 
